//
// Generic Dialog functions, called from the menu and generic commands
//
// Author: J. van Wijk
//
// JvW  26-01-2005 Initial version
//

#include <txlib.h>                              // TX library interface

#include <dfsdisk.h>                            // FS disk structure defs
#include <dfspart.h>                            // FS partition info manager
#include <dfsupart.h>                           // FS partition utilities
#include <dfsector.h>                           // DFS store functions
#include <dfs.h>                                // DFS navigation and defs
#include <dfsutil.h>                            // DFS utility functions
#include <dfswin.h>                             // window and help definitions
#include <dfsdgen.h>                            // generic dialogs interface

//- Window styles used with the Dialog Widgets
//- DFSDM is 'moving' DFSDS' is sizing and DFSD2 is half-sizing, all horizontal

#define DFSDMAUTOCHK  TXWS_AUTOCHK | TXWS_HCHILD_MOVE
#define DFSDMAUTORAD  TXWS_AUTORAD | TXWS_HCHILD_MOVE
#define DFSDMPBUTTON  TXWS_PBUTTON | TXWS_HCHILD_MOVE
#define DFSDMOUTPUT   TXWS_OUTPUT  | TXWS_HCHILD2MOVE
#define DFSD2OUTPUT   TXWS_OUTPUT  | TXWS_HCHILD2SIZE
#define DFSDSOUTPUT   TXWS_OUTPUT  | TXWS_HCHILD_SIZE
#define DFSDSENTRYB   TXWS_ENTRYB  | TXWS_HCHILD_SIZE
#define DFSDSLISTBOX  TXWS_DSPINB  | TXLS_DROP_ENTER | TXWS_LEFTJUSTIFY | TXWS_HCHILD_SIZE

static TXTM        fdescr = "";                 // FROM object/store description
static TXTM        tdescr = "";                 // TO   object/store description


/*========================== FIND ===============================================================*/
static BOOL fdwr1  = TRUE;                      // Radio Every sector
static BOOL fdwr2  = FALSE;                     // Check Cylinder boundaries
static BOOL fdwr3  = FALSE;                     // Radio Freespace only
static BOOL fdwr4  = FALSE;                     // Radio Allocated only

static BOOL fdwr5  = TRUE;                      // Radio Ascii
static BOOL fdwr6  = FALSE;                     // Radio Unicode
static BOOL fdwr7  = FALSE;                     // Radio Hex
static BOOL fdwr8  = FALSE;                     // Radio Mixed

static BOOL fdwrS  = FALSE;                     // Radio Single shot
static BOOL fdwrR  = TRUE;                      // Radio Repeat search
static BOOL fdwrM  = FALSE;                     // Radio Multiple hits/sector

static BOOL fdwc1  = FALSE;                     // Check Case-sensitive
static BOOL fdwc2  = TRUE;                      // Check No sector span
static BOOL fdwc3  = FALSE;                     // Check Search backward
static BOOL fdwc4  = FALSE;                     // Check NOT containing
static BOOL fdwc5  = FALSE;                     // Check Verbose output
static BOOL fdwc6  = FALSE;                     // Check Start at NEXT
static BOOL fdwc7  = TRUE;                      // Show search args

static TXTS fdwe1  = "";                        // position for 1st argument
static TXTS fdwe2  = "";                        // sector types
static TXTM fdwe3  = "";                        // string for 2nd argument

/*
() Ascii          () Repeat, 1 hit/sector  () Search in every sector
( ) Unicode        ( ) Repeat multiple hits  ( ) On Cylinder boundaries
( ) Hex pairs      ( ) Search once, display  ( ) In freespace (undelete)
( ) Mixed string   [ ] Case-sensitive match  ( ) In allocated (filegrep)
[ ] Verbose output [ ] Search backwards      [ ] Start at NEXT/PREV sect
[] Show arguments [ ] NOT containing ...    [] No sector span (faster)

 @Position   Types   Find ... AND 2nd string, 2nd is case-sensitive!
[.........] [.....] [..................................................]
*/
#define   DFSDGFINDWIDGETS 21
TXWIDGET  dfsDgFindWidgets[DFSDGFINDWIDGETS] =  // order determines TAB-order!
{
   {0,  0, 1, 18, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr5, "Ascii")},
   {1,  0, 1, 18, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr6, "Unicode")},
   {2,  0, 1, 18, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr7, "Hex pairs")},
   {3,  0, 1, 18, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr8, "Mixed string")},
   {4,  0, 1, 18, 0, 5, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc5, "Verbose output")},
   {5,  0, 1, 18, 0, 5, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc7, "Show arguments")},

   {0, 19, 1, 25, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( fdwrR, "Repeat, 1 hit/sector")},
   {1, 19, 1, 25, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( fdwrM, "Repeat multiple hits")},
   {2, 19, 1, 25, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( fdwrS, "Search once, display")},
   {3, 19, 1, 25, 0, 6, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc1, "Case-sensitive match")},
   {4, 19, 1, 25, 0, 6, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc3, "Search backwards")},
   {5, 19, 1, 25, 0, 6, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc4, "NOT containing ...")},


   {0, 45, 1, 28, 0, 3, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr1, "Search in every sector" )},
   {1, 45, 1, 28, 0, 3, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr2, "On Cylinder boundaries" )},
   {2, 45, 1, 28, 0, 3, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr3, "In freespace (undelete)")},
   {3, 45, 1, 28, 0, 3, 0, TXWS_AUTORAD, 0, TXStdButton( fdwr4, "In allocated (filegrep)")},
   {4, 45, 1, 28, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc6, "Start at NEXT/PREV sect")},
   {5, 45, 1, 28, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( fdwc2, "No sector span (faster)")},

   {7,  0, 2, 11, 0, 0, 0, TXWS_ENTRYBT, 0, TXStdEntryf( fdwe1, TXMAXTS, "@Position")},
   {7, 12, 2,  7, 0, 0, 0, TXWS_ENTRYBT, 0, TXStdEntryf( fdwe2, TXMAXTS, "Types")},
   {7, 20, 2, 52, 0, 0, 0, TXWS_ENTRYBT  | TXWS_HCHILD_SIZE, 0,
              TXStdEntryf( fdwe3, TXMAXTM, "Find ... AND 2nd string, 2nd is case-sensitive!")}
};

TXGW_DATA dfsDgFindDlg =
{
   DFSDGFINDWIDGETS,                            // number of widgets
   DFSC_FINDTC,                                 // dialog help
   810,                                         // base window ID
   &dfsDgFindWidgets                            // array of widgets
};

static TXLN   findarg1 = "";                    // first search argument


/*************************************************************************************************/
// Present FIND options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsFindDialog
(
   void
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;

   ENTER();

   dfsBEGINWORK();                              // signal work starting
   while (txwPromptBox( TXHWND_DESKTOP, TXHWND_DESKTOP, &dfsDgFindDlg,
         "Specify the primary ASCII, UNICODE, HEX or Mixed string "
         "to search the sectors for. Adjust the search options to "
         "your liking, for usage consult online-help with <F1> or "
         "the 'FIND' command documentation\n\n"
         "The search will start at the CURRENT sector (this)",
         " Specify search string(s) and options ", DFSC_FINDTC,
         TXPB_MOVEABLE | TXPB_HCENTER | TXPB_VCENTER, 72, findarg1)
      != TXDID_CANCEL)
   {
      if (strlen(findarg1) || strlen(fdwe3) || strlen(fdwe2))
      {
         strcpy(             command, "find -o:");
         if (!fdwc1) strcat( command, "$");
         if ( fdwc2) strcat( command, "%");
         if ( fdwc3) strcat( command, "-");
         if ( fdwc4) strcat( command, "^");
         if ( fdwc5) strcat( command, "+");     // optional verbose for repeat
         if ( fdwrS) strcat( command, "+");     // single, always verbose
         if ( fdwrR) strcat( command, "*");
         if ( fdwrM) strcat( command, "*!M");
         if ( fdwr2) strcat( command, "~");
         if (!fdwc7) strcat( command, "!");
         if ( fdwr3) strcat( command, "[");
         if ( fdwr4) strcat( command, "]");

         if ((strlen(fdwe1) != 0) && (isdigit(fdwe1[0]))) // valid position
         {
            strcat( command, "@");
            strcat( command, fdwe1);
         }
         if (fdwc6)
         {
            strcat( command, " -n");
         }
         if (strlen(fdwe2) != 0)                // sector types specified
         {
            strcat( command, " -t:'");
            strcat( command, fdwe2);
            strcat( command, "'");
         }
         if (strlen(fdwe3))                     // 2nd search string
         {
            if ( fdwr5) strcat( command, " -A:\"");
            if ( fdwr6) strcat( command, " -U:\"");
            if ( fdwr7) strcat( command, " -H:\"");
            if ( fdwr8) strcat( command, " -M:\"");
            strcat(             command, fdwe3);
            strcat(             command, "\"");
         }
         if (strlen(findarg1))                  // 1st search string
         {
            if ( fdwr5) strcat( command, " -a:\"");
            if ( fdwr6) strcat( command, " -u:\"");
            if ( fdwr7) strcat( command, " -h:\"");
            if ( fdwr8) strcat( command, " -m:\"");
            strcat(             command, findarg1);
            strcat(             command, "\"");
         }
         dfsExecEnd( command);
         break;                                 // out of while ...
      }
      else
      {
         TxMessage( TRUE, DFSC_FINDTC,
                    "You must specify at least one search "
                    "argument or a sector Type ...");
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsFindDialog'
/*-----------------------------------------------------------------------------------------------*/


/*========================== IMPORT =============================================================*/
static BOOL impdwc1 = FALSE;                    // Include data sectors

/*
[ ] Import each DATA sector from .BIN/.BN? file too
*/
#define   DFSDGIMPWIDGETS 1
static TXWIDGET  dfsDgImpWidgets[DFSDGIMPWIDGETS] = // order determines TAB-order!
{
   {0,  0, 1, 58, 0, 0, 0, TXWS_AUTOCHK, 0, TXStdButton( impdwc1,
                          "Import each DATA sector from .BIN/.BN? file too")}
};

static TXGW_DATA dfsDgImpDlg =
{
   DFSDGIMPWIDGETS,                             // number of widgets
   DFSC_IMPLIST,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgImpWidgets                             // array of widgets
};

/*************************************************************************************************/
// Present Import options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsImportDialog
(
   void
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;

   ENTER();

   dfsBEGINWORK();                              // signal work starting
   strcpy( fspec, "*.sn?;*.lsn");
   if (txwOpenFileDialog( fspec, NULL, NULL, DFSC_IMPLIST, &dfsDgImpDlg,
       " Select .SNx or .LSN sector-list file to be imported ", fspec))
   {
      sprintf(    command, "import %s%s", fspec, (impdwc1) ? " -d" : "");
      dfsExecCmd( command);
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsImportDialog'
/*-----------------------------------------------------------------------------------------------*/


/*========================== EXPORT =============================================================*/
static BOOL expdwc1 = FALSE;                    // Include data sectors

/*
[ ] Export each DATA sector to a .BIN/.BN? file too
*/
#define   DFSDGEXPWIDGETS 1
static TXWIDGET  dfsDgExpWidgets[DFSDGEXPWIDGETS] = // order determines TAB-order!
{
   {0,  0, 1, 58, 0, 0, 0, TXWS_AUTOCHK, 0, TXStdButton( expdwc1,
                          "Export each DATA sector to a .BIN/.BN? file too")}
};

static TXGW_DATA dfsDgExpDlg =
{
   DFSDGEXPWIDGETS,                             // number of widgets
   DFSC_EXPLIST,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgExpWidgets                             // array of widgets
};

/*************************************************************************************************/
// Present Export options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsExportDialog
(
   void
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;

   ENTER();

   dfsBEGINWORK();                              // signal work starting
   strcpy( fspec, "*.sn?;*.lsn");
   if (txwOpenFileDialog( fspec, NULL, NULL, DFSC_EXPLIST, &dfsDgExpDlg,
       " Specify file for exporting the sector-list to ", fspec))
   {
      sprintf(    command, "export %s%s", fspec, (expdwc1) ? " -d" : "");
      dfsExecCmd( command);
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsExportDialog'
/*-----------------------------------------------------------------------------------------------*/


/*========================== SIM ================================================================*/
static BOOL        simr1  = FALSE;              // Single image-file
static BOOL        simr2  = TRUE;               // Multi file, max size
static BOOL        simr3  = FALSE;              // Multi file removable

static BOOL        simr4  = FALSE;              // RAW image
static BOOL        simr5  = FALSE;              // Compressed
static BOOL        simr6  = TRUE;               // Smart compr

static BOOL        simc1  = FALSE;              // Append to file
static BOOL        simc2  = FALSE;              // Ignore errors
static BOOL        simc3  = FALSE;              // Exclude LVMsig
static BOOL        simc4  = TRUE;               // HEX sector values
static BOOL        simc5  = FALSE;              // Verbose, debug mode

static TXTS        sime1  = "2047";             // Multiple max size
static BOOL        simrk  = FALSE;              // KiB
static BOOL        simrm  = TRUE;               // MiB
static BOOL        simrg  = FALSE;              // GiB

static TXTS        sime2  = "";                 // From
static TXTS        sime3  = "";                 // Size
static BOOL        simrK  = FALSE;              // KiB
static BOOL        simrM  = FALSE;              // MiB
static BOOL        simrG  = FALSE;              // GiB
static BOOL        simrS  = TRUE;               // Sec

/*
 First sector   Image size  [] Sector values are HEX, not decimal
[0...........] [Whole Area] () Sectors  ( ) KiB  ( ) MiB  ( ) GiB
Multi only: max size per file: [650.....]( ) KiB  () MiB  ( ) GiB

 From Partition:[04 ECS12       F: Log 07 HPFS        917.7 MiB ]

[ ] Append to file    () Single file, any size     ( ) RAW image
[ ] Ignore errors     ( ) Multi file, on one disk   ( ) Compressed
[ ] Exclude LVM area  ( ) Multi file, media change  () Smart comp
[ ] Verbose list data, debug mode
*/
#define   DFSDGSIMWIDGETS 25
#define   DFSDGSIMT1      12                    // index of description
#define   DFSDGSIMT2      13                    // index of leader
#define   DFSDGSIML1      14                    // index of list-1
TXWIDGET  dfsDgSimWidgets[DFSDGSIMWIDGETS] =    // order determines TAB-order!
{
   {0, 28, 1, 38, 0, 0, 0, DFSDMAUTOCHK, 0, TXStdButton( simc4, "Sector values are HEX, not decimal")},
   {0,  0, 2, 14, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE,
                                         0, TXStdEntryf( sime2, TXMAXTS, "First sector")},
   {0, 15, 2, 12, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE |  TXWS_HCHILD2MOVE,
                                         0, TXStdEntryf( sime3, TXMAXTS, "Image size")},

   {1, 28, 1, 11, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( simrS, "Sectors")},
   {1, 41, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( simrK, "KiB")},
   {1, 50, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( simrM, "MiB")},
   {1, 59, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( simrG, "GiB")},

   {2,  1, 1, 30, 0, 0, 0, DFSD2OUTPUT,  0, TXStdStline( "Multi only: max size per file:")},
   {2, 31, 1, 10, 0, 0, 0, TXWS_ENTRYB   |  TXWS_HCHILD2SIZE |  TXWS_HCHILD2MOVE,
                                         0, TXStdEntryf( sime1, TXMAXTS, "")},
   {2, 41, 1,  7, 0, 5, 0, DFSDMAUTORAD, 0, TXStdButton( simrk, "KiB")},
   {2, 50, 1,  7, 0, 5, 0, DFSDMAUTORAD, 0, TXStdButton( simrm, "MiB")},
   {2, 59, 1,  7, 0, 5, 0, DFSDMAUTORAD, 0, TXStdButton( simrg, "GiB")},

   {4, 17, 1, 50, 1, 0, 1, DFSDSOUTPUT,  0, TXStdStline( fdescr)},
   {4,  1, 1, 15, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( NULL)},
   {4, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {6,  0, 1, 24, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( simc1, "Append to file")},
   {7,  0, 1, 24, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( simc2, "Ignore errors")},
   {8,  0, 1, 24, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( simc3, "Exclude LVM area")},
   {9,  0, 1, 40, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( simc5, "Verbose list data, debug mode")},

   {6, 22, 1, 29, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( simr1, "Single file, any size")},
   {7, 22, 1, 29, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( simr2, "Multi file, on one disk")},
   {8, 22, 1, 29, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( simr3, "Multi file, media change")},

   {6, 52, 1, 15, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( simr4, "RAW image")},
   {7, 52, 1, 15, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( simr5, "Compressed")},
   {8, 52, 1, 15, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( simr6, "Smart comp")}
};

TXGW_DATA dfsDgSimDlg =
{
   DFSDGSIMWIDGETS,                             // number of widgets
   DFSC_IMAGDLG,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgSimWidgets                             // array of widgets
};


/*************************************************************************************************/
// Present SIM options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsSimDialog
(
   DFSOBJECT           source,                  // IN    Source for the image
   char               *image,                   // IN    filename, "" or NULL
   BOOL                media,                   // IN    set media-change
   ULONG               mfsize,                  // IN    max filesize or 0
   ULONG               tsector,                 // IN    To   sector number
   ULONG               sizetodo                 // IN    Size to clone (0 = all)
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;
   TXTT                option;
   TXSELIST           *list = NULL;
   TXTM                text;

   ENTER();

   TRACES(( "IMAGE to:'%s'  mfsize: %lu start position:%8.8lx  size:%8.8lx\n",
            (image) ? image : "NULL", mfsize, tsector, sizetodo));

   dfsBEGINWORK();                              // signal work starting

   if (media)
   {
      simr1 = FALSE;
      simr2 = FALSE;
      simr3 = TRUE;                             // force media-change set
   }
   if (mfsize != 0)                             // maximum filesize specified
   {
      sprintf( sime1, "%lu", mfsize);           // value in ASCII
      simrk  = FALSE;                           // KiB
      simrm  = TRUE;                            // MiB
      simrg  = FALSE;                           // GiB
   }
   sprintf( sime2, "%lx", tsector);             // set start sector number
   if (sizetodo != 0)
   {
      sprintf( sime3, "%lx", sizetodo);         // set Size to process (0 = all)
   }
   else
   {
      strcpy(  sime3, "Whole Area");
   }

   dfsDgSimWidgets[DFSDGSIMT1].flags  |=  TXWI_DISABLED; // disable static
   dfsDgSimWidgets[DFSDGSIML1].flags  &= ~TXWI_DISABLED; // enable list
   switch (source)
   {
      case DFSO_VOLD:
         TRACES(( "Volume, list: %8.8lx\n", dfsa->slVolumes));
         dfsDgSimWidgets[DFSDGSIMT2].sl.buf = "From    volume:";
         dfsDgSimWidgets[DFSDGSIML1].title  = " Create image from volume ";
         if (dfsa->slVolumes == NULL)
         {
            dfsa->slVolumes = TxFsDriveSelist( TXFSV_HD | TXFSV_CD |
                                               TXFSV_FLOP, FALSE);
         }
         list = dfsa->slVolumes;
         TxSelStringSelect( list,  SINF->drive, 2);
         break;

      case DFSO_DISK:
         TRACES(( "Disk, list: %8.8lx\n", dfsa->slDiskOne));
         dfsDgSimWidgets[DFSDGSIMT2].sl.buf = "From      disk:";
         dfsDgSimWidgets[DFSDGSIML1].title  = " Create image from disk ";
         list = dfsa->slDiskOne;
         sprintf( text, "%2hu",    SINF->disknr);
         TxSelStringSelect( list,  text, 2);
         break;

      case DFSO_PART:
         TRACES(( "Disk, list: %8.8lx\n", dfsa->slPartOne));
         dfsDgSimWidgets[DFSDGSIMT2].sl.buf = "From partition:";
         dfsDgSimWidgets[DFSDGSIML1].title  = " Create image from partition ";
         list = dfsa->slPartOne;
         sprintf( text, "%02.2hu", SINF->partid);
         TxSelStringSelect( list,  text, 2);
         break;

      default:
         if (source == DFSO_IMGF)
         {
            strcpy( fdescr, dfstStoreDesc2( DFSTORE) + 10);
            dfsDgSimWidgets[DFSDGSIMT2].sl.buf  = "From imagefile:";
         }
         else                                   // other types
         {
            strcpy( fdescr, dfstStoreDesc1( DFSTORE) + 10);
            dfsDgSimWidgets[DFSDGSIMT2].sl.buf  = "From   current:";
         }
         dfsDgSimWidgets[DFSDGSIMT1].flags  &= ~TXWI_DISABLED;
         dfsDgSimWidgets[DFSDGSIML1].flags  |=  TXWI_DISABLED;
         break;
   }
   dfsDgSimWidgets[DFSDGSIML1].lb.list = list;

   strcpy( fspec, "*.i??");
   while (txwSaveAsFileDialog( fspec, NULL, image, DFSC_IMAGDLG, &dfsDgSimDlg,
      " Specify imaging options and name for (1st) imagefile ", fspec))
   {
      if ((fspec[strlen(fspec)-1] != FS_PATH_SEP) && // not a directory ?
                (strlen(fspec) == TxStrWcnt(fspec))) // and no wildcard ?
      {
         sprintf( command, "image %s %s", fspec, (strlen(sime2)) ? sime2 : "0");
         if (!simc4)
         {
            strcat( command, ",d");             // make 'First' decimal
         }
         if ((strlen( sime3) != 0) &&           // Size value specified
             (isxdigit( sime3[0]))  )           // and it is a number
         {
            TxStrip( sime3, sime3, ' ', ' ');
            sprintf( option, " %s,%s", sime3, (simrK) ? "k" :
                                              (simrM) ? "m" :
                                              (simrG) ? "g" :
                                              (simc4) ? "s" :
                                                        "d");
            strcat(  command, option);
         }
         if (list)
         {
            strcpy(  text, list->items[ list->selected]->text);
            TxStrip( text, text, ' ', ' ');     // strip leading spaces
            TxRepl(  text, ' ', 0);             // replace next space by 0
            switch (source)                     // select proper source
            {
               case DFSO_VOLD: sprintf( option, " -V:'%s'", text);    break;
               case DFSO_DISK: sprintf( option, " -D:%s",   text);    break;
               case DFSO_PART: sprintf( option, " -P:%s",   text);    break;
               default:        strcpy(  option, "");                  break;
            }
            strcat( command, option);
         }
         if (!simr1)                            // not a single file
         {
            strcat( command, " -m:");
            if (strlen( sime1) != 0)            // size specified
            {
               TxStrip( sime1, sime1, ' ', ' ');
               sprintf( option, "%s,%s", sime1, (simrk) ? "k" :
                                                (simrm) ? "m" :
                                                          "g");
               strcat( command, option);
            }
         }
         if ( simc1)         strcat( command, " -A");
         if ( simc2)         strcat( command, " -E:i");
         if ( simc3)         strcat( command, " -L");
         if ( simc5)         strcat( command, " -v");
         if ( simr4)         strcat( command, " -R");
         if ( simr5 | simr6) strcat( command, " -z");
         if ( simr6)         strcat( command, " -S");
         if ( simr3)         strcat( command, " -M");

         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_IMAGING, "You must specify an image filename, "
                                        "not a wildcard or directory ...");
         strcpy(  fspec, "*.i??");
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsSimDialog'
/*-----------------------------------------------------------------------------------------------*/



/*========================== WRIM ===============================================================*/
static BOOL wrimr1  = TRUE;                     // Restore
static BOOL wrimr2  = FALSE;                    // Verify
static BOOL wrimr3  = FALSE;                    // Compare

static BOOL wrimc2  = FALSE;                    // Exclude, -Lvm
static BOOL wrimc4  = TRUE;                     // HEX sector values
static BOOL wrimc5  = FALSE;                    // Verbose, debug mode
static BOOL wrimc6  = TRUE;                     // Use size from IMZ header

static TXTS wrime2  = "";                       // From
static TXTS wrime3  = "";                       // Size
static BOOL wrimrK  = FALSE;                    // KiB
static BOOL wrimrM  = FALSE;                    // MiB
static BOOL wrimrG  = FALSE;                    // GiB
static BOOL wrimrS  = TRUE;                     // Sec

/*
 First sector   Image size  [] Sector values are HEX, not decimal
[0...........] [Whole Area] () Sectors  ( ) KiB  ( ) MiB  ( ) GiB

 To Partition : [04 ECS12       F: Log 07 HPFS        917.7 MiB ]

[ ] Exclude LVM area               () Restore, write to object
[ ] Verbose list data, debug mode  ( ) Compare imagefile to object
[ ] Use size from IMZ header       ( ) Verify imagefile integrity
*/
#define   DFSDGWRIMWIDGETS 16
#define   DFSDGWRIMT1       7                   // index of description
#define   DFSDGWRIMT2       8                   // index of leader
#define   DFSDGWRIML1       9                   // index of list-1
TXWIDGET  dfsDgWrimWidgets[DFSDGWRIMWIDGETS] =  // order determines TAB-order!
{
   {0, 28, 1, 38, 0, 0, 0, DFSDMAUTOCHK, 0, TXStdButton( wrimc4, "Sector values are HEX, not decimal")},
   {0,  0, 2, 14, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE,
                                         0, TXStdEntryf( wrime2, TXMAXTS, "First sector")},
   {0, 16, 2, 12, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE |  TXWS_HCHILD2MOVE,
                                         0, TXStdEntryf( wrime3, TXMAXTS, "Image size")},
   {1, 28, 1, 11, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( wrimrS, "Sectors")},
   {1, 41, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( wrimrK, "KiB")},
   {1, 50, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( wrimrM, "MiB")},
   {1, 59, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( wrimrG, "GiB")},

   {3, 17, 1, 50, 1, 0, 0, DFSDSOUTPUT,  0, TXStdStline( tdescr)},
   {3,  1, 1, 15, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( NULL)},
   {3, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {5,  0, 1, 34, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( wrimc2, "Exclude LVM area")},
   {6,  0, 1, 34, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( wrimc5, "Verbose list data, debug mode")},
   {7,  0, 1, 34, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( wrimc6, "Use size from IMZ header")},

   {5, 35, 1, 32, 0, 2, 0, DFSDMAUTORAD, 0, TXStdButton( wrimr1, "Restore, write to object")},
   {6, 35, 1, 32, 0, 2, 0, DFSDMAUTORAD, 0, TXStdButton( wrimr3, "Compare imagefile to object")},
   {7, 35, 1, 32, 0, 2, 0, DFSDMAUTORAD, 0, TXStdButton( wrimr2, "Verify imagefile integrity")}
};

TXGW_DATA dfsDgWrimDlg =
{
   DFSDGWRIMWIDGETS,                            // number of widgets
   DFSC_WRIMAGE,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgWrimWidgets                            // array of widgets
};


/*************************************************************************************************/
// Present WRIM options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsWrimDialog
(
   DFSOBJECT           destination,             // IN    Destination for image
   char               *image,                   // IN    filename, "" or NULL
   BOOL                verify,                  // IN    verify only
   ULONG               tsector,                 // IN    To   sector number
   ULONG               sizetodo                 // IN    Size to clone (0 = all)
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;
   TXTT                option;
   TXTM                title;
   TXGW_DATA          *widgets = NULL;
   TXTS                wildcard;
   TXSELIST           *list    = NULL;
   TXTM                text;

   ENTER();
   TRACES(( "%s  from:'%s'  start position:%8.8lx  size:%8.8lx\n",
          (verify) ? "RESTORE" : "VERIFY ", tsector, sizetodo));

   dfsBEGINWORK();                              // signal work starting

   sprintf( wrime2, "%lx", tsector);            // set start sector number
   if (sizetodo != 0)
   {
      sprintf( wrime3, "%lx", sizetodo);        // set Size to process (0 = all)
   }
   else
   {
      strcpy(  wrime3, "Whole Area");
   }

   if (verify)                                  // VERIFY only
   {
      strcpy( title, " Select an imagefile to be checked for file damage ");
      strcpy( wildcard, "*.imz");
   }
   else
   {
      strcpy( title, " Select an imagefile and options for the Restore/Compare ");
      switch (destination)
      {
         case DFSO_VOLD:
            dfsDgWrimWidgets[DFSDGWRIMT2].sl.buf = "To     volume :";
            dfsDgWrimWidgets[DFSDGWRIML1].title  = " Restore image to volume ";
            if (dfsa->slVolumes == NULL)
            {
               dfsa->slVolumes = TxFsDriveSelist( TXFSV_HD | TXFSV_CD |
                                                  TXFSV_FLOP, FALSE);
            }
            list = dfsa->slVolumes;
            TxSelStringSelect( list,  SINF->drive, 2);
            break;

         case DFSO_DISK:
            dfsDgWrimWidgets[DFSDGWRIMT2].sl.buf = "To       disk :";
            dfsDgWrimWidgets[DFSDGWRIML1].title  = " Restore image to disk ";
            list = dfsa->slDiskOne;
            sprintf( text, "%2hu",    SINF->disknr);
            TxSelStringSelect( list,  text, 2);
            break;

         case DFSO_PART:
            dfsDgWrimWidgets[DFSDGWRIMT2].sl.buf = "To  partition :";
            dfsDgWrimWidgets[DFSDGWRIML1].title  = " Restore image to partition ";
            list = dfsa->slPartOne;
            sprintf( text, "%02.2hu", SINF->partid);
            TxSelStringSelect( list,  text, 2);
            break;

         default:
            if (destination == DFSO_IMGF)
            {
               strcpy( tdescr, dfstStoreDesc2( DFSTORE) + 10);
               dfsDgWrimWidgets[DFSDGWRIMT2].sl.buf  = "To  imagefile :";
            }
            else                                // other types
            {
               strcpy( tdescr, dfstStoreDesc1( DFSTORE) + 10);
               dfsDgWrimWidgets[DFSDGWRIMT2].sl.buf  = "To    current :";
            }
            dfsDgWrimWidgets[DFSDGWRIMT1].flags  &= ~TXWI_DISABLED;
            dfsDgWrimWidgets[DFSDGWRIML1].flags  |=  TXWI_DISABLED;
            break;
      }
      dfsDgWrimWidgets[DFSDGWRIML1].lb.list = list;
      strcpy( wildcard, "*.i??");
      widgets = &dfsDgWrimDlg;
   }

   strcpy( fspec, wildcard);
   while (txwOpenFileDialog( fspec, NULL, image, DFSC_WRIMAGE, widgets, title, fspec))
   {
      if ((fspec[strlen(fspec)-1] != FS_PATH_SEP) && // not a directory ?
                (strlen(fspec) == TxStrWcnt(fspec))) // and no wildcard ?
      {
         sprintf( command, "restore %s %s", fspec, (strlen(wrime2)) ? wrime2 : "0");
         if (!wrimc4)
         {
            strcat( command, ",d");             // make 'First' decimal
         }
         if ((strlen( wrime3) != 0) &&          // Size value specified
             (isxdigit( wrime3[0]))  )          // and it is a number
         {
            TxStrip( wrime3, wrime3, ' ', ' ');
            sprintf( option, " %s,%s", wrime3, (wrimrK) ? "k" :
                                               (wrimrM) ? "m" :
                                               (wrimrG) ? "g" :
                                               (wrimc4) ? "s" :
                                                          "d");
            strcat(  command, option);
            wrimc6 = FALSE;                     // ignore 'size from IMZ'
         }
         if (list)
         {
            strcpy(  text, list->items[ list->selected]->text);
            TxStrip( text, text, ' ', ' ');     // strip leading spaces
            TxRepl(  text, ' ', 0);             // replace next space by 0
            switch (destination)                // select proper destination
            {
               case DFSO_VOLD: sprintf( option, " -V:'%s'", text);    break;
               case DFSO_DISK: sprintf( option, " -D:%s",   text);    break;
               case DFSO_PART: sprintf( option, " -P:%s",   text);    break;
               default:        strcpy(  option, "");                  break;
            }
            strcat( command, option);
         }
         if ( wrimr2 | verify) strcat( command, " -test");
         if ( wrimr3)          strcat( command, " -c");
         if ( wrimc2)          strcat( command, " -L");
         if ( wrimc5)          strcat( command, " -v");
         if ( wrimc6)          strcat( command, " -S");

         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_WRIMAGE, "You must specify an image filename, "
                                        "not a wildcard or directory ...");
         strcpy( fspec, wildcard);
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsWrimDialog'
/*-----------------------------------------------------------------------------------------------*/


/*========================== CLONE ==============================================================*/
static BOOL        cloc1  = FALSE;              // Merge on error
static BOOL        cloc2  = FALSE;              // Ignore errors
static BOOL        cloc3  = FALSE;              // Exclude LVMsig
static BOOL        cloc4  = TRUE;               // HEX sector values
static BOOL        cloc5  = FALSE;              // COMPARE only

static TXTS        cloe1  = "";                 // From sector
static TXTS        cloe2  = "";                 // To   sector
static TXTS        cloe3  = "";                 // Size
static BOOL        clorK  = FALSE;              // KiB
static BOOL        clorM  = FALSE;              // MiB
static BOOL        clorG  = FALSE;              // GiB
static BOOL        clorS  = TRUE;               // Sec
static BOOL        cloOK  = TRUE;               // OK button, dummy variable
static BOOL        cloCA  = FALSE;              // Cancel button dummy

/*
[ ] VERIFY only, compare FROM against TO    [ ] Ignore errors
[ ] Merge original data on read errors      [ ] Exclude LVM area

 From Partition:[04 ECS12       F: Log 07 HPFS        917.7 MiB ]

 To   Partition:[17                Log 07 -           917.7 MiB ]

                                                       Ŀ
     First sector                                        OK  
To  [............]                                     

     First sector  Size to do [] Sector values are HEX, not decimal
From[............][Whole Area]() Sectors  ( ) KiB  ( ) MiB  ( ) GiB
*/
#define   DFSDGCLOWIDGETS 23
#define   DFSDGCLOD1       4                    // index of from description
#define   DFSDGCLOT1       5                    // index of from leader
#define   DFSDGCLOL1       6                    // index of from list-1
#define   DFSDGCLOD2       7                    // index of to description
#define   DFSDGCLOT2       8                    // index of to leader
#define   DFSDGCLOL2       9                    // index of to list-2
#define   DFSDGCLOOK      10                    // index of OK button
TXWIDGET  dfsDgCloWidgets[DFSDGCLOWIDGETS] =    // order determines TAB-order!
{
   {0,  0, 1, 42, 0, 6, 0, TXWS_AUTOCHK, 0, TXStdButton( cloc5, "COMPARE only, verify FROM against TO")},
   {0, 44, 1, 24, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( cloc2, "Ignore errors")},
   {1,  0, 1, 42, 0, 6, 0, TXWS_AUTOCHK, 0, TXStdButton( cloc1, "Merge original data on read errors")},
   {1, 44, 1, 24, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( cloc3, "Exclude LVM area")},

   {4, 17, 1, 50, 1, 0, 0, DFSDSOUTPUT,  0, TXStdStline( fdescr)},
   {4,  0, 1, 15, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( NULL)},
   {4, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {7, 17, 1, 50, 1, 0, 0, DFSDSOUTPUT,  0, TXStdStline( tdescr)},
   {7,  0, 1, 15, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( NULL)},
   {7, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {9, 45, 3,  8, 0, 0, TXDID_OK,
                           DFSDMPBUTTON, 0, TXStdButton( cloOK, " OK ")},
   {9, 57, 3,  8, 0, 0, TXDID_CANCEL,
                           DFSDMPBUTTON, 0, TXStdButton( cloCA, "Cancel")},

   {10, 4, 2, 14, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE,
                                         0, TXStdEntryf( cloe1, TXMAXTS, "First sector")},
   {11, 0, 1,  4, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( "From")},

   {13, 4, 2, 14, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE,
                                         0, TXStdEntryf( cloe2, TXMAXTS, "First sector")},
   {13,18, 2, 12, 0, 0, 0, TXWS_ENTRYBT  |  TXWS_HCHILD2SIZE |  TXWS_HCHILD2MOVE,
                                         0, TXStdEntryf( cloe3, TXMAXTS, "Size to do")},
   {13,30, 1, 38, 0, 0, 0, DFSDMAUTOCHK, 0, TXStdButton( cloc4, "Sector values are HEX, not decimal")},
   {14, 0, 1,  4, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( "To")},

   {14,30, 1, 11, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( clorS, "Sectors")},
   {14,43, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( clorK, "KiB")},
   {14,52, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( clorM, "MiB")},
   {14,61, 1,  7, 0, 4, 0, DFSDMAUTORAD, 0, TXStdButton( clorG, "GiB")}
};

TXGW_DATA dfsDgCloDlg =
{
   DFSDGCLOWIDGETS,                             // number of widgets
   DFSC_CLONDLG,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgCloWidgets                             // array of widgets
};


/*************************************************************************************************/
// Present CLONE options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsCloneDialog
(
   BOOL                clone,                   // IN    Clone, not VERIFY
   DFSOBJECT           fobject,                 // IN    From object reference
   ULONG               fsector,                 // IN    From sector number
   DFSOBJECT           tobject,                 // IN    To   object reference
   ULONG               tsector,                 // IN    To   sector number
   ULONG               sizetodo                 // IN    Size to clone (0 = all)
)
{
   ULONG               rc = NO_ERROR;           // function return
   ULONG               fstart = 0;              // from start
   ULONG               tstart = 0;              // to   start
   ULONG               fselected;               // from selected
   ULONG               tselected;               // to   selected
   USHORT              focus;                   // field to get focus
   TXSELIST           *flist  = NULL;           // from list
   TXSELIST           *tlist  = NULL;           // to   list
   DFST_HANDLE         tstore = DFSTORE;        // DFSee STORE ids
   DFST_HANDLE         fstore = (tstore == 1) ? 2 : 1;
   DFSISTORE          *fsi;                     // FROM store info struct
   TXLN                command;
   TXTT                option;
   TXTM                text;

   ENTER();
   TRACES(( "%s  from:%8.8lx  at:%8.8lx  size:%8.8lx\n",
          (clone) ? "CLONE " : "VERIFY", fsector, tsector, sizetodo));

   dfsBEGINWORK();                              // signal work starting

   if (dfstQueryStoreType( fstore, &fsi, NULL) == DFST_UNUSED)
   {
      fsi = SINF;                               // when not set yet, default to
   }                                            // same as the CURRENT to-store
   dfsDgCloWidgets[DFSDGCLOD1].flags  |=  TXWI_DISABLED; // disable static
   dfsDgCloWidgets[DFSDGCLOL1].flags  &= ~TXWI_DISABLED; // enable list
   switch (fobject)
   {
      case DFSO_VOLD:
         dfsDgCloWidgets[DFSDGCLOT1].sl.buf = "From    volume:";
         dfsDgCloWidgets[DFSDGCLOL1].title  = " Clone from volume ";
         if (dfsa->slVolumes == NULL)
         {
            dfsa->slVolumes = TxFsDriveSelist( TXFSV_HD | TXFSV_CD |
                                               TXFSV_FLOP, FALSE);
         }
         rc = TxSelistDuplicate( dfsa->slVolumes, FALSE, TRUE, &flist);
         TxSelStringSelect( flist,  fsi->drive, 2);
         break;

      case DFSO_DISK:
         dfsDgCloWidgets[DFSDGCLOT1].sl.buf = "From      disk:";
         dfsDgCloWidgets[DFSDGCLOL1].title  = " Clone from disk ";
         rc = TxSelistDuplicate( dfsa->slDiskOne, FALSE, TRUE, &flist);
         sprintf( text, "%2hu",     fsi->disknr);
         TxSelStringSelect( flist,  text, 2);
         break;

      case DFSO_PART:
         dfsDgCloWidgets[DFSDGCLOT1].sl.buf = "From partition:";
         dfsDgCloWidgets[DFSDGCLOL1].title  = " Clone from partition ";
         rc = TxSelistDuplicate( dfsa->slPartOne, FALSE, TRUE, &flist);
         sprintf( text, "%02.2hu",  fsi->partid);
         TxSelStringSelect( flist,  text, 2);
         break;

      default:
         if (fobject == DFSO_IMGF)
         {
            strcpy( fdescr, dfstStoreDesc2( fstore) + 10);
            dfsDgCloWidgets[DFSDGCLOT1].sl.buf  = "From imagefile:";
         }
         else                                   // other types
         {
            strcpy( fdescr, dfstStoreDesc1( fstore) + 10);
            dfsDgCloWidgets[DFSDGCLOT1].sl.buf  = "From   current:";
         }
         dfsDgCloWidgets[DFSDGCLOD1].flags  &= ~TXWI_DISABLED;
         dfsDgCloWidgets[DFSDGCLOL1].flags  |=  TXWI_DISABLED;
         break;
   }
   dfsDgCloWidgets[DFSDGCLOL1].lb.list = flist;

   dfsDgCloWidgets[DFSDGCLOD2].flags  |=  TXWI_DISABLED; // disable static
   dfsDgCloWidgets[DFSDGCLOL2].flags  &= ~TXWI_DISABLED; // enable list
   switch (tobject)
   {
      case DFSO_VOLD:
         dfsDgCloWidgets[DFSDGCLOT2].sl.buf = "To      volume:";
         dfsDgCloWidgets[DFSDGCLOL2].title  = " Clone to volume ";
         if (dfsa->slVolumes == NULL)
         {
            dfsa->slVolumes = TxFsDriveSelist( TXFSV_HD | TXFSV_CD |
                                               TXFSV_FLOP, FALSE);
         }
         tlist = dfsa->slVolumes;
         TxSelStringSelect( tlist,  SINF->drive, 2);
         break;

      case DFSO_DISK:
         dfsDgCloWidgets[DFSDGCLOT2].sl.buf = "To        disk:";
         dfsDgCloWidgets[DFSDGCLOL2].title  = " Clone to disk ";
         tlist = dfsa->slDiskOne;
         sprintf( text, "%2hu",    SINF->disknr);
         TxSelStringSelect( tlist,  text, 2);
         break;

      case DFSO_PART:
         dfsDgCloWidgets[DFSDGCLOT2].sl.buf = "To   partition:";
         dfsDgCloWidgets[DFSDGCLOL2].title  = " Clone to partition ";
         tlist = dfsa->slPartOne;
         sprintf( text, "%02.2hu", SINF->partid);
         TxSelStringSelect( tlist,  text, 2);
         break;

      default:
         if (tobject == DFSO_IMGF)
         {
            strcpy( tdescr, dfstStoreDesc2( tstore) + 10);
            dfsDgCloWidgets[DFSDGCLOT2].sl.buf  = "To   imagefile:";
         }
         else                                   // other types
         {
            strcpy( tdescr, dfstStoreDesc1( tstore) + 10);
            dfsDgCloWidgets[DFSDGCLOT2].sl.buf  = "To     current:";
         }
         dfsDgCloWidgets[DFSDGCLOD2].flags  &= ~TXWI_DISABLED;
         dfsDgCloWidgets[DFSDGCLOL2].flags  |=  TXWI_DISABLED;
         break;
   }
   dfsDgCloWidgets[DFSDGCLOL2].lb.list = tlist;

   if      (flist != NULL) focus = DFSDGCLOL1;
   else if (tlist != NULL) focus = DFSDGCLOL2;
   else                    focus = DFSDGCLOOK;

   cloc5 = (clone == FALSE);                    // set COMPARE checkbox
   cloc4 = TRUE;
   sprintf( cloe1, "%lx", fsector);
   sprintf( cloe2, "%lx", tsector);             // set From/To sector numbers
   if (sizetodo != 0)
   {
      sprintf( cloe3, "%lx", sizetodo);         // set Size to clone  (0 = all)
   }
   else
   {
      strcpy( cloe3, "Whole Area");
   }

   while ((rc == NO_ERROR) && (txwWidgetDialog( TXHWND_DESKTOP, TXHWND_DESKTOP,
           NULL, " Specify cloning options and 'From' plus 'To' objects ",
           TXWD_MOVEABLE | TXWD_HCENTER | TXWD_VCENTER, focus, &dfsDgCloDlg)
        != TXDID_CANCEL))
   {
      sscanf( cloe1, "%lx", &fstart);
      sscanf( cloe2, "%lx", &tstart);            // get to/from start value

      fselected = (flist) ? flist->selected : 11111; // different values, always
      tselected = (tlist) ? tlist->selected : 22222; // larger that list sizes

      if ((fobject != tobject) || (fselected != tselected) || (fstart != tstart))
      {
         sprintf( command, "%s %s", (cloc5)         ? "comp" : "clone",
                                    (strlen(cloe2)) ?  cloe2 : "0");
         if (!cloc4)
         {
            strcat( command, ",d");             // make 'First' decimal
         }
         if ((strlen( cloe3) != 0) &&           // Size value specified
             (isxdigit( cloe3[0]))  )           // and it is a number
         {
            TxStrip( cloe3, cloe3, ' ', ' ');
            sprintf( option, " %s,%s", cloe3, (clorK) ? "k" :
                                              (clorM) ? "m" :
                                              (clorG) ? "g" :
                                              (cloc4) ? "s" :
                                                        "d");
            strcat(  command, option);
         }

         if ((strlen( cloe1) != 0)     &&       // From start sector
             (strcmp( cloe1, "0") != 0) )       // and not zero
         {
            TxStrip( cloe1, cloe1, ' ', ' ');
            sprintf( option, " -f:%s,%s", cloe1, (clorK) ? "k" :
                                                 (clorM) ? "m" :
                                                 (clorG) ? "g" :
                                                 (cloc4) ? "s" :
                                                           "d");
            strcat(  command, option);
         }

         if (flist)
         {
            strcpy(  text, flist->items[ flist->selected]->text);
            TxStrip( text, text, ' ', ' ');     // strip leading spaces
            TxRepl(  text, ' ', 0);             // replace next space by 0
            switch (fobject)                    // select proper source
            {
               case DFSO_VOLD: sprintf( option, " -v:'%s'", text);    break;
               case DFSO_DISK: sprintf( option, " -d:%s",   text);    break;
               case DFSO_PART: sprintf( option, " -p:%s",   text);    break;
               default:        strcpy(  option, "");                  break;
            }
            strcat( command, option);
         }

         if (tlist)
         {
            strcpy(  text, tlist->items[ tlist->selected]->text);
            TxStrip( text, text, ' ', ' ');     // strip leading spaces
            TxRepl(  text, ' ', 0);             // replace next space by 0
            switch (tobject)                    // select proper destination
            {
               case DFSO_VOLD: sprintf( option, " -V:'%s'", text);    break;
               case DFSO_DISK: sprintf( option, " -D:%s",   text);    break;
               case DFSO_PART: sprintf( option, " -P:%s",   text);    break;
               default:        strcpy(  option, "");                  break;
            }
            strcat( command, option);
         }

         if ( cloc1)         strcat( command, " -M");
         if ( cloc2)         strcat( command, " -E:i");
         if ( cloc3)         strcat( command, " -L");

         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_CLONING, "The selected FROM and TO objects/First sector seem "
                                        "to be the same!\n\nYou must specify a 'FROM' "
                                        "location that is DIFFERENT from the 'TO' location "
                                        "to be able to clone between them ...");
      }
   }
   if (flist)                                   // free Duplicated list
   {
      txSelDestroy( &flist);
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsCloneDialog'
/*-----------------------------------------------------------------------------------------------*/



/*========================== GENPART ============================================================*/
static BOOL        genc1  = FALSE;              // Sector based size/location
static BOOL        genc2  = FALSE;              // No location/freespace

static TXTM        gene1  = "";                 // Description

/*
    From   disk:  1 Maxtor 114.5 GiB      117239.9 MiB          ]

    Description:[                                                ]

[ ] Use sector based sizes and location, for exact copy on any GEO
[ ] Do NOT include freespace areas, use default location on create

    Specify base filename only, path is added, extension stripped
*/
#define   DFSDGGENWIDGETS 7
#define   DFSDGGENL1      1                     // index of list-1
TXWIDGET  dfsDgGenWidgets[DFSDGGENWIDGETS] =    // order determines TAB-order!
{
   {0,  4, 1, 12, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( "From   disk:")},
   {0, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {2,  4, 1, 12, 0, 0, 0, DFSDSOUTPUT,  0, TXStdStline( "Description:")},
   {2, 16, 1, 50, 0, 0, 0, TXWS_ENTRYB   |  TXWS_HCHILD2SIZE |  TXWS_HCHILD2MOVE,
                                         0, TXStdEntryf( gene1, TXMAXTM, "")},

   {4,  0, 1, 66, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( genc1,
                          "Use sector based sizes and location, for exact copy on any GEO")},
   {5,  0, 1, 66, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( genc2,
                          "Do NOT include freespace areas, use default location on create")},

   {7,  4, 1, 62, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline(
                          "Specify base filename only, path is added, extension stripped")},
};

TXGW_DATA dfsDgGenDlg =
{
   DFSDGGENWIDGETS,                             // number of widgets
   DFSC_GENPDLG,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgGenWidgets                             // array of widgets
};


/*************************************************************************************************/
// Present GENPART options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsGenPartDialog
(
   USHORT              disk,                    // IN    disk number string
   char               *base,                    // IN    base filename
   BOOL                sectors,                 // IN    use sector values
   BOOL                nofree                   // IN    no freespace
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;
   TXSELIST           *list = NULL;
   ULONG               item = 0;                // default to ALL disks

   ENTER();

   dfsBEGINWORK();                              // signal work starting

   genc1 = sectors;
   genc2 = nofree;

   list = dfsa->slDiskAll;
   if ((disk != 0) && (disk != FDSK_ANY))
   {
      sprintf( command, "%2hu", disk);
      TxSelStringSelect( list, command, 2);
   }
   else
   {
      list->selected = 0;                       // default to 'all disks'
   }
   dfsDgGenWidgets[DFSDGGENL1].lb.list = list;
   dfsDgGenWidgets[DFSDGGENL1].title   = " Generate partition-script for disk ";

   strcpy( fspec, "*.dfs");
   while (txwSaveAsFileDialog( fspec, NULL, base, DFSC_GENPDLG, &dfsDgGenDlg,
      " Specify GENPART options and basename for the scriptfile ", fspec))
   {
      if ((fspec[strlen(fspec)-1] != FS_PATH_SEP) && // not a directory ?
                (strlen(fspec) == TxStrWcnt(fspec))) // and no wildcard ?
      {
         if (list)                              // get selected disk nr
         {
            item = list->items[ list->selected]->value -TXDID_MAX;
         }
         sprintf( command, "genpart %lu %s ", item, fspec);
         if ( genc1)         strcat( command, "-s ");
         if ( genc2)         strcat( command, "-f- ");
         if (strlen( gene1) != 0)              // Description specified
         {
            strcat(  command, gene1);
         }
         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_GENPDLG, "You must specify a script base filename, "
                                        "not a wildcard or directory ...");
         strcpy(  fspec, "*.dfs");
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsGenPartDialog'
/*-----------------------------------------------------------------------------------------------*/



/*========================== DFSDISK ============================================================*/
static BOOL        dskr1  = FALSE;              // Search ALL sectors (slow!)
static BOOL        dskr2  = TRUE;               // Search cylinder boundaries
static BOOL        dskr3  = FALSE;              // Partition, BR and LVM only
static BOOL        dskr4  = FALSE;              // Include HPFS   superblocks
static BOOL        dskr5  = FALSE;              // Include NTFS   NTLDR sectors
static BOOL        dskr6  = FALSE;              // Include JFS    superblocks
static BOOL        dskr7  = TRUE;               // Include EXT2   superblocks
static BOOL        dskr8  = FALSE;              // Include REISER superblocks

static TXTS        dske1  = "calculated";       // #Cylinders
static TXTS        dske2  = "current";          // #Heads
static TXTS        dske3  = "current";          // #Sectors

/*
   Analyse disk:  1 Maxtor 114.5 GiB      117239.9 MiB          ]

() Partition, BR and LVM only   ( ) Search ALL sectors (slow!)
( ) Include HPFS   superblocks   () Search cylinder boundaries
( ) Include NTFS   NTLDR sectors
( ) Include JFS    superblocks    #  Cyls.   [calculated]
( ) Include EXT2   superblocks    #  Heads   [current   ] 1..255
( ) Include REISER superblocks    #  Sectors [current   ] 1..63

    Specify base filename only, path is added, extension stripped
*/
#define   DFSDGDSKWIDGETS 19
#define   DFSDGDSKL1      1                     // index of list-1
TXWIDGET  dfsDgDskWidgets[DFSDGDSKWIDGETS] =    // order determines TAB-order!
{
   {0,  3, 1, 13, 0, 0, 0, TXWS_OUTPUT,  0, TXStdStline( "Analyse disk:")},
   {0, 16, 1, 50, 0, 0, 0, DFSDSLISTBOX, 0, TXStdLstBox( NULL, 0, NULL)},

   {2,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr3, "Partition, BR and LVM only")},
   {3,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr4, "Include HPFS   superblocks")},
   {4,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr5, "Include NTFS   NTLDR sectors")},
   {5,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr6, "Include JFS    superblocks")},
   {6,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr7, "Include EXT2   superblocks")},
   {7,  1, 1, 32, 0, 1, 0, TXWS_AUTORAD, 0, TXStdButton( dskr8, "Include REISER superblocks")},

   {2, 33, 1, 30, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( dskr1, "Search ALL sectors (slow!)")},
   {3, 33, 1, 30, 0, 2, 0, TXWS_AUTORAD, 0, TXStdButton( dskr2, "Search cylinder boundaries")},

   {5, 34, 1, 11, 0, 3, 0, TXWS_OUTPUT,  0, TXStdStline( "#  Cyls.")},
   {5, 45, 1, 12, 0, 3, 0, DFSDSENTRYB,  0, TXStdEntryf( dske1, TXMAXTS, "")},
   {6, 34, 1, 11, 0, 3, 0, TXWS_OUTPUT,  0, TXStdStline( "#  Heads")},
   {6, 45, 1, 12, 0, 3, 0, DFSDSENTRYB,  0, TXStdEntryf( dske2, TXMAXTS, "")},
   {6, 58, 1,  8, 0, 3, 0, DFSDMOUTPUT,  0, TXStdStline( "1..255")},
   {7, 34, 1, 11, 0, 3, 0, TXWS_OUTPUT,  0, TXStdStline( "#  Sectors")},
   {7, 45, 1, 12, 0, 3, 0, DFSDSENTRYB,  0, TXStdEntryf( dske3, TXMAXTS, "")},
   {7, 58, 1,  8, 0, 3, 0, DFSDMOUTPUT,  0, TXStdStline( "1..63")},

   {9,  4, 1, 62, 0, 0, 0, DFSD2OUTPUT,  0, TXStdStline(
                          "Specify base filename only, path is added, extension stripped")},
};

TXGW_DATA dfsDgDskDlg =
{
   DFSDGDSKWIDGETS,                             // number of widgets
   DFSC_DISKDLG,                                // help, widget overrules
   810,                                         // base window ID
   &dfsDgDskWidgets                             // array of widgets
};


/*************************************************************************************************/
// Present DFSDISK options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsDfsDiskDialog
(
   USHORT              disk,                    // IN    disk number string
   char               *base                     // IN    base filename
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;
   TXSELIST           *list = NULL;
   ULONG               item = 0;                // default to ALL disks

   ENTER();

   dfsBEGINWORK();                              // signal work starting

   list = dfsa->slDiskAll;
   if ((disk != 0) && (disk != FDSK_ANY))
   {
      sprintf( command, "%2hu", disk);
      TxSelStringSelect( list, command, 2);
   }
   else
   {
      list->selected = 0;                       // default to 'all disks'
   }
   dfsDgDskWidgets[DFSDGDSKL1].lb.list = list;
   dfsDgDskWidgets[DFSDGDSKL1].title   = " Generate DFSDISK analysis for disk ";

   strcpy( fspec, "*.sk?");
   while (txwSaveAsFileDialog( fspec, NULL, base, DFSC_DISKDLG, &dfsDgDskDlg,
      " Specify DFSDISK options and basename for the report files ", fspec))
   {
      if ((fspec[strlen(fspec)-1] != FS_PATH_SEP) && // not a directory ?
                (strlen(fspec) == TxStrWcnt(fspec))) // and no wildcard ?
      {
         if (list)                              // get selected disk nr
         {
            item = list->items[ list->selected]->value -TXDID_MAX;
         }
         sprintf( command, "dfsdisk %lu -b:'%s' ", item, fspec);
         if      ( dskr4)           strcat( command, "-s:HPFS ");
         if      ( dskr5)           strcat( command, "-s:NTFS ");
         if      ( dskr6)           strcat( command, "-s:JFS ");
         if      ( dskr7)           strcat( command, "-s:EXT2 ");
         if      ( dskr8)           strcat( command, "-s:REISER ");
         if      ( dskr1)
         {
            strcat( command, "ALL");            // ALL sectors, very slow
         }
         else
         {
            strcat( command, "CYL ");           // CYL, optional GEO stuff
            if ((dske1[0] != 'c') || (dske2[0] != 'c') || (dske3[0] != 'c'))
            {
               strcat( command, (dske1[0] != 'c') ? dske1 : "?");
               strcat( command, " ");
               strcat( command, (dske2[0] != 'c') ? dske2 : "");
               strcat( command, " ");
               strcat( command, (dske3[0] != 'c') ? dske3 : "");
            }
         }
         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_DISKDLG, "You must specify a report base filename, "
                                        "not a wildcard or directory ...");
         strcpy(  fspec, "*.sk?");
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsDfsDiskDialog'
/*-----------------------------------------------------------------------------------------------*/

/*========================== LOG ================================================================*/
static TXLN        logo1;                       // Output field user message
static BOOL        logc1;                       // Log-reopen at each line

/*
  Specify a filename on a WRITABLE volume/driveletter for logging
  All new screen output will be APPENDED to the specified file.")

  Create logfile for the session, or use Cancel/Esc for no logging

[ ] Close and re-open the logfile after writing each line (slow!)
*/
#define   DFSDGLOGWIDGETS 4
TXWIDGET  dfsDgLogWidgets[DFSDGLOGWIDGETS] =    // order determines TAB-order!
{
   {0,  2, 1, 65, 0, 0, 1, DFSDSOUTPUT,  0, TXStdStline( fdescr)},
   {1,  2, 1, 65, 0, 0, 1, DFSDSOUTPUT,  0, TXStdStline( tdescr)},
   {3,  2, 1, 65, 0, 0, 1, DFSDSOUTPUT,  0, TXStdStline( logo1)},

   {5,  0, 1, 65, 0, 7, 0, TXWS_AUTOCHK, 0, TXStdButton( logc1,
   "Close and reopen file after writing each line (slow!)")},
};

TXGW_DATA dfsDgLogDlg =
{
   DFSDGLOGWIDGETS,                             // number of widgets
   DFSC_LOGF,                                   // help, widget overrules
   810,                                         // base window ID
   &dfsDgLogWidgets                             // array of widgets
};


/*************************************************************************************************/
// Present LOG options dialog and execute resulting command
/*************************************************************************************************/
ULONG dfsLogDialog
(
   char               *logname,                 // IN    default name or NULL
   char               *message,                 // IN    extra message or NULL
   BOOL                reopen                   // IN    reopen logfile
)
{
   ULONG               rc = NO_ERROR;           // function return
   TXLN                command;
   TXLN                fspec;

   ENTER();

   dfsBEGINWORK();                              // signal work starting

   logc1 = reopen;

   strcpy( fdescr, "Specify a filename on a WRITABLE volume/driveletter for logging");
   strcpy( tdescr, "All new screen output will be APPENDED to the specified file.");

   if (message && strlen( message))
   {
      strcpy( logo1, message);
   }
   else
   {
      strcpy( logo1, "Start logfile for the session, or use Cancel/Esc for no logging");
   }

   strcpy( fspec, "*.log");
   while (txwSaveAsFileDialog( fspec, NULL, logname, DFSC_LOGF, &dfsDgLogDlg,
      " Specify filename for logging this session to ", fspec))
   {
      if ((fspec[strlen(fspec)-1] != FS_PATH_SEP) && // not a directory ?
                (strlen(fspec) == TxStrWcnt(fspec))) // and no wildcard ?
      {
         sprintf( command, "log '%s'", fspec);  // allow spaces in path/name!
         if (logc1)
         {
            strcat( command, " -r");            // reopen option
         }
         dfsExecEnd( command);
         break;
      }
      else
      {
         TxMessage( TRUE, DFSC_LOGF, "You must specify a log filename, "
                                     "not a wildcard or directory ...");
         strcpy(  fspec, "*.log");
      }
   }
   dfsENDWORK();                                // signal work done
   RETURN (rc);
}                                               // end 'dfsLogDialog'
/*-----------------------------------------------------------------------------------------------*/



