// TxLib filesystem functions, Critical-error handling
//

#include <txlib.h>                              // TxLib interface
#include <txtpriv.h>                            // private interface


#if defined (DOS32)
   ULONG            TxAutoFail   = 1;           // start in AutoFail for DOS
#else
   ULONG            TxAutoFail   = 0;           // but NOT for OS/2 and WIN!
#endif                                          // to assure first autofail
                                                // call to succeeed


/*****************************************************************************/
// Set critical error handling to auto-fail (versus interactive Abort, retry .)
// Calls to this may be nested, autofail is ON until matching fail==false call
/*****************************************************************************/
void TxFsAutoFailCriticalErrors
(
   BOOL                fail                     // IN    autofail on CritErr
)
{
   ENTER();
   TRACES(( "TxAutoFail: %lu, fail: %s\n", TxAutoFail, (fail) ? "TRUE" : "NO"));

   if ((fail == FALSE) && (TxAutoFail != 0))
   {
      if (--TxAutoFail == 0)                    // 0 after decrement
      {
         #if defined   (WIN32)
            SetErrorMode( 0);
         #elif defined (DEV32)
            DosError( FERR_ENABLEHARDERR);
         #endif
      }
   }
   else if (fail == TRUE)
   {
      if (TxAutoFail++ == 0)                    // 0 before increment
      {
         #if defined   (WIN32)
            SetErrorMode( SEM_FAILCRITICALERRORS);
         #elif defined (DEV32)
            DosError( FERR_DISABLEHARDERR);
         #endif
      }
   }
   //- Note: DOS TX-handler will obey the TxAutoFail level directly ...
   VRETURN();
}                                           // end 'TxFsAutoFailCriticalErrors'
/*---------------------------------------------------------------------------*/

