// Screen output using ANSI X3.64 terminal escape definitions and
// file-logging facilities
//
// Author: J. van Wijk
//
// JvW  24-01-2003 Removed thread-id from default hex-dump display
// JvW  01-02-2002 Added 7 bit ASCII filtering using switch '-7'
// JvW  10-12-2001 Added TxStrListAdd
// JvW  29-08-2001 Rename to TXCON
// JvW  17-01-2000 Added UNICODE modus to FormatMixedStr function
// JvW  08-03-1998 Added WIN32 ANSI expansion function (JvD)
// JvW  20-04-1997 Ported to WIN32
// JvW  27-01-1996 Made more generic, for LPTIO tests
// JvW  15-02-1996 Added ddl global debugging variable
// JvW  09-03-1996 Added trace macro's and implementation
// JvW  26-12-1996 Added RetBool support for 64 threads
// JvW  17-06-1995 Initial version, split off from DHPFS.C

#include <txlib.h>
#include <txwpriv.h>                            // private window interface
#include <txtpriv.h>                            // private text   interface

static  SCREEN_STATE   screen_act  = SCREEN_ON;

#if defined (USEWINDOWING)
static  char txConfirmTitle[] = " Confirmation request ";
static  char txMsgWarnTitle[] = " Notification or warning message ";
#endif

                                                // Hook control structures
        TXH_INFO   *txh_clean = NULL;           // clean TxPrint handler
        TXH_INFO   *txh_raw   = NULL;           // raw ansi TxPrint handler

static  char       *txm_buff  = NULL;           // temporary buffer for TxPrint

static  BOOL        txk_Abort = FALSE;          // TX KB-system abort notify
static  ULONG       txkThread = 0;              // TX KB-system owning thread

#define ANSI_S_TIMEOUT  10000                   // 10 seconds timeout on claim

static  BOOL  txc_ascii7 = FALSE;               // ASCII filtering off
static  char  txc_ansi   = A_ON;


//- Note: 256 color strings sorted in default PC order
ANSIDEFS  ansi =
{
   "[0;40;30m",
   "[0;40;34m",
   "[0;40;32m",
   "[0;40;36m",
   "[0;40;31m",
   "[0;40;35m",
   "[0;40;33m",
   "[0;40;37m",
   "[1;40;30m",
   "[1;40;34m",
   "[1;40;32m",
   "[1;40;36m",
   "[1;40;31m",
   "[1;40;35m",
   "[1;40;33m",
   "[1;40;37m",
   "[0;44;30m",
   "[0;44;34m",
   "[0;44;32m",
   "[0;44;36m",
   "[0;44;31m",
   "[0;44;35m",
   "[0;44;33m",
   "[0;44;37m",
   "[1;44;30m",
   "[1;44;34m",
   "[1;44;32m",
   "[1;44;36m",
   "[1;44;31m",
   "[1;44;35m",
   "[1;44;33m",
   "[1;44;37m",
   "[0;42;30m",
   "[0;42;34m",
   "[0;42;32m",
   "[0;42;36m",
   "[0;42;31m",
   "[0;42;35m",
   "[0;42;33m",
   "[0;42;37m",
   "[1;42;30m",
   "[1;42;34m",
   "[1;42;32m",
   "[1;42;36m",
   "[1;42;31m",
   "[1;42;35m",
   "[1;42;33m",
   "[1;42;37m",
   "[0;46;30m",
   "[0;46;34m",
   "[0;46;32m",
   "[0;46;36m",
   "[0;46;31m",
   "[0;46;35m",
   "[0;46;33m",
   "[0;46;37m",
   "[1;46;30m",
   "[1;46;34m",
   "[1;46;32m",
   "[1;46;36m",
   "[1;46;31m",
   "[1;46;35m",
   "[1;46;33m",
   "[1;46;37m",
   "[0;41;30m",
   "[0;41;34m",
   "[0;41;32m",
   "[0;41;36m",
   "[0;41;31m",
   "[0;41;35m",
   "[0;41;33m",
   "[0;41;37m",
   "[1;41;30m",
   "[1;41;34m",
   "[1;41;32m",
   "[1;41;36m",
   "[1;41;31m",
   "[1;41;35m",
   "[1;41;33m",
   "[1;41;37m",
   "[0;45;30m",
   "[0;45;34m",
   "[0;45;32m",
   "[0;45;36m",
   "[0;45;31m",
   "[0;45;35m",
   "[0;45;33m",
   "[0;45;37m",
   "[1;45;30m",
   "[1;45;34m",
   "[1;45;32m",
   "[1;45;36m",
   "[1;45;31m",
   "[1;45;35m",
   "[1;45;33m",
   "[1;45;37m",
   "[0;43;30m",
   "[0;43;34m",
   "[0;43;32m",
   "[0;43;36m",
   "[0;43;31m",
   "[0;43;35m",
   "[0;43;33m",
   "[0;43;37m",
   "[1;43;30m",
   "[1;43;34m",
   "[1;43;32m",
   "[1;43;36m",
   "[1;43;31m",
   "[1;43;35m",
   "[1;43;33m",
   "[1;43;37m",
   "[0;47;30m",
   "[0;47;34m",
   "[0;47;32m",
   "[0;47;36m",
   "[0;47;31m",
   "[0;47;35m",
   "[0;47;33m",
   "[0;47;37m",
   "[1;47;30m",
   "[1;47;34m",
   "[1;47;32m",
   "[1;47;36m",
   "[1;47;31m",
   "[1;47;35m",
   "[1;47;33m",
   "[1;47;37m",

   "[5;40;30m",
   "[5;40;34m",
   "[5;40;32m",
   "[5;40;36m",
   "[5;40;31m",
   "[5;40;35m",
   "[5;40;33m",
   "[5;40;37m",
   "[5;40;1;30m",
   "[5;40;1;34m",
   "[5;40;1;32m",
   "[5;40;1;36m",
   "[5;40;1;31m",
   "[5;40;1;35m",
   "[5;40;1;33m",
   "[5;40;1;37m",
   "[5;44;30m",
   "[5;44;34m",
   "[5;44;32m",
   "[5;44;36m",
   "[5;44;31m",
   "[5;44;35m",
   "[5;44;33m",
   "[5;44;37m",
   "[5;44;1;30m",
   "[5;44;1;34m",
   "[5;44;1;32m",
   "[5;44;1;36m",
   "[5;44;1;31m",
   "[5;44;1;35m",
   "[5;44;1;33m",
   "[5;44;1;37m",
   "[5;42;30m",
   "[5;42;34m",
   "[5;42;32m",
   "[5;42;36m",
   "[5;42;31m",
   "[5;42;35m",
   "[5;42;33m",
   "[5;42;37m",
   "[5;42;1;30m",
   "[5;42;1;34m",
   "[5;42;1;32m",
   "[5;42;1;36m",
   "[5;42;1;31m",
   "[5;42;1;35m",
   "[5;42;1;33m",
   "[5;42;1;37m",
   "[5;46;30m",
   "[5;46;34m",
   "[5;46;32m",
   "[5;46;36m",
   "[5;46;31m",
   "[5;46;35m",
   "[5;46;33m",
   "[5;46;37m",
   "[5;46;1;30m",
   "[5;46;1;34m",
   "[5;46;1;32m",
   "[5;46;1;36m",
   "[5;46;1;31m",
   "[5;46;1;35m",
   "[5;46;1;33m",
   "[5;46;1;37m",
   "[5;41;30m",
   "[5;41;34m",
   "[5;41;32m",
   "[5;41;36m",
   "[5;41;31m",
   "[5;41;35m",
   "[5;41;33m",
   "[5;41;37m",
   "[5;41;1;30m",
   "[5;41;1;34m",
   "[5;41;1;32m",
   "[5;41;1;36m",
   "[5;41;1;31m",
   "[5;41;1;35m",
   "[5;41;1;33m",
   "[5;41;1;37m",
   "[5;45;30m",
   "[5;45;34m",
   "[5;45;32m",
   "[5;45;36m",
   "[5;45;31m",
   "[5;45;35m",
   "[5;45;33m",
   "[5;45;37m",
   "[5;45;1;30m",
   "[5;45;1;34m",
   "[5;45;1;32m",
   "[5;45;1;36m",
   "[5;45;1;31m",
   "[5;45;1;35m",
   "[5;45;1;33m",
   "[5;45;1;37m",
   "[5;43;30m",
   "[5;43;34m",
   "[5;43;32m",
   "[5;43;36m",
   "[5;43;31m",
   "[5;43;35m",
   "[5;43;33m",
   "[5;43;37m",
   "[5;43;1;30m",
   "[5;43;1;34m",
   "[5;43;1;32m",
   "[5;43;1;36m",
   "[5;43;1;31m",
   "[5;43;1;35m",
   "[5;43;1;33m",
   "[5;43;1;37m",
   "[5;47;30m",
   "[5;47;34m",
   "[5;47;32m",
   "[5;47;36m",
   "[5;47;31m",
   "[5;47;35m",
   "[5;47;33m",
   "[5;47;37m",
   "[5;47;1;30m",
   "[5;47;1;34m",
   "[5;47;1;32m",
   "[5;47;1;36m",
   "[5;47;1;31m",
   "[5;47;1;35m",
   "[5;47;1;33m",
   "[5;47;1;37m",

   "[0m",                                      // NORMAL
   "[1A",                                      // CURSOR_UP1
   "[2A",                                      // CURSOR_UP2
   "[4A",                                      // CURSOR_UP4
   "[8A",                                      // CURSOR_UP8
   "[1B",                                      // CURSOR_DOWN1
   "[2B",                                      // CURSOR_DOWN2
   "[4B",                                      // CURSOR_DOWN4
   "[8B",                                      // CURSOR_DOWN8
   "[1C",                                      // CURSOR_RIGHT1
   "[2C",                                      // CURSOR_RIGHT2
   "[4C",                                      // CURSOR_RIGHT4
   "[8C",                                      // CURSOR_RIGHT8
   "[1D",                                      // CURSOR_LEFT1
   "[2D",                                      // CURSOR_LEFT2
   "[4D",                                      // CURSOR_LEFT4
   "[8D",                                      // CURSOR_LEFT8
   "[s",                                       // CURSOR_SAVEP
   "[u",                                       // CURSOR_RESTP
   "[K",                                       // CLEAR_TO_EOL
   "[1;70H",                                   // CURS_GO_1_70
   "[1;75H",                                   // CURS_GO_1_75
   "[1;1H",                                    // CURS_GO_1_01
   "[2;1H",                                    // CURS_GO_2_01
   "[23;1H",                                   // CURS_GO23_01
   "[24;1H"                                    // CURS_GO24_01
};


/*****************************************************************************/
// Set 7-bit ASCII mode
/*****************************************************************************/
void TxSetAscii7Mode
(
   BOOL                mode                     // IN    ASCII 7-bit mode
)
{
   TRACES(( "ASCII7 set 1: %s\n", (mode) ? "TRUE" : "FALSE"));
   txc_ascii7 = mode;
}                                               // end 'TxSetAscii7Mode'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Get ASCII mode, 7-bit or full
/*****************************************************************************/
BOOL TxGetAscii7Mode                              // RET   ASCII 7-bit in use
(
   void
)
{
   return( txc_ascii7);
}                                               // end 'TxGetAscii7Mode'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Set ansi string definitions active or inactive
/*****************************************************************************/
void TxSetAnsiMode
(
   char                mode                     // IN    ansi mode
)
{
   int                 i;

   txc_ansi = mode;
   for (i = 0; i < NUMBER_ANSIS; i++)
   {
      ansi[i][0] = mode;
   }
}                                               // end 'TxSetAnsiMode'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Get ansi string definitions active or inactive
/*****************************************************************************/
char TxGetAnsiMode                              // RET   ansi mode
(
   void
)
{
   return( txc_ansi);
}                                               // end 'TxGetAnsiMode'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Test and set TxScreenState
/*****************************************************************************/
SCREEN_STATE TxScreenState                      // RET   screen active
(
   SCREEN_STATE        action                   // IN    screen action
)
{
   if (action != SCREEN_TEST)
   {
      screen_act = action;
   }
   return (screen_act);
}                                               // end 'TxScreenState'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// printf-like print on stdout, LOG filehandle and raw/clean copy hooks
/*****************************************************************************/
void TxPrint
(
   char               *fmt,                     // IN    format string (printf)
   ...                                          // IN    variable arguments
)
{
   va_list             varargs;
   ULONG               size = 0;
   BOOL                log7bit;
   BOOL                logreopen;
   FILE               *log_handle = TxQueryLogFile( &log7bit, &logreopen);
   TXH_INFO           *hinfo;                   // current hook info

   va_start(varargs, fmt);
   if (txm_buff == NULL)
   {
      txm_buff = calloc( 1, TXMAX4K);
   }
   if (txm_buff != NULL)
   {
      size = vsprintf( txm_buff, fmt, varargs); // expanded string in buffer

      if (txc_ascii7)                           // filter to 7-bit ASCII
      {
         TxAscii827( txm_buff, TXASCII827_TRANS);
      }

      if ((screen_act    == SCREEN_ON) &&       // screen output ON
          (txh_raw       == NULL))              // and not redirected to a RAW
      {                                         // stream (like scrollbuffer)
         #if defined (WIN32)
            txNtConsoleDisplay( txm_buff);      // ANSI expand and print
         #else
            printf( "%s", txm_buff);            // just print formatted buffer
         #endif
         fflush(stdout);
      }
      for (hinfo = txh_raw; hinfo != NULL; hinfo = hinfo->next)
      {
         if ((size < hinfo->size) && (hinfo->active == TRUE) &&
             ((screen_act == SCREEN_ON) ||
              (hinfo->follow_screen_toggle == FALSE)))
         {
            strcpy( hinfo->cbuf, txm_buff);
            (hinfo->copy) (hinfo->cbuf, hinfo->user);
         }
      }
      if ((log_handle != 0) || (txh_clean != NULL))
      {
         TxStripAnsiCodes( txm_buff);
         for (hinfo = txh_clean; hinfo != NULL; hinfo = hinfo->next)
         {
            if ((size < hinfo->size) && (hinfo->active == TRUE) &&
                ((screen_act == SCREEN_ON) ||
                 (hinfo->follow_screen_toggle == FALSE)))
            {
               strcpy( hinfo->cbuf, txm_buff);
               (hinfo->copy) (hinfo->cbuf, hinfo->user);
            }
         }
         if (log_handle != 0)
         {
            if (!txc_ascii7 && log7bit)         // 7-bit ASCII for log only ?
            {
               TxAscii827( txm_buff, TXASCII827_TRANS);
            }
            fprintf( log_handle, "%s", txm_buff);
            if (logreopen)
            {
               TxCloseReopenLogFile();          // close/reopen (force flush)
            }
            else
            {
               fflush(  log_handle);            // soft flush
            }
         }
      }
   }
   va_end(varargs);
}                                               // end 'TxPrint'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Remove ANSI control code from a text-string (in place conversion)
/*****************************************************************************/
char *TxStripAnsiCodes                          // RET   stripped string
(
   char               *text                     // INOUT ANSI text to strip
)
{
   char               *rd = text;               // read-pointer in string
   char               *wr = text;               // write pointer

   while (*rd)
   {
      if ((*rd == TXK_ESCAPE) && (*(rd+1) == '['))
      {
         rd += 2;
         while ((isdigit(*rd)) || (*rd == ';'))
         {
            rd++;
         }
      }
      else
      {
         *(wr++) = *rd;                         // copy the character
      }
      rd++;
   }
   *wr = '\0';                                  // terminate stripped string
   return( text);
}                                               // end 'TxStripAnsiCodes'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Claim TX keyboard access-functions for current thread (TxAbort etc)
/*****************************************************************************/
ULONG TxClaimKeyboard                           // RET   previous owner
(
   void
)
{
   ULONG               rc = txkThread;          // function return

   ENTER();

   txkThread = TXCURTHREAD;

   RETURN (rc);
}                                               // end 'TxClaimKeyboard'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Cancel any pending abort status (as returned by TxAbort)
/*****************************************************************************/
void TxCancelAbort
(
   void
)
{
   ENTER();

   txk_Abort = FALSE;

   VRETURN ();
}                                               // end 'TxCancelAbort'
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
// Set abort status to pending, for other threads (as returned by TxAbort)
/*****************************************************************************/
void TxSetPendingAbort
(
   void
)
{
   ENTER();

   txk_Abort = TRUE;

   VRETURN ();
}                                               // end 'TxSetPendingAbort'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Return kbhit status and latest More ... reply, read away the hit
/*****************************************************************************/
BOOL TxAbort                                    // RET   TRUE when abort wanted
(
   void
)
{
   BOOL                aborting = FALSE;
   ULONG               key;

   if ((txk_Abort) ||                           // already aborting
       (txkThread != TXCURTHREAD))              // or not KB owning thread
   {
      aborting = txk_Abort;
   }
   else
   {
      if (kbhit())
      {
         #if defined (USEWINDOWING)
            key = txwGetInputEvent( NULL);
         #else
            key = getch();
         #endif
         switch (key)
         {
            case TXK_ESCAPE:
               aborting  = TRUE;                // abort if <Esc>
               txk_Abort = TRUE;                // remember abort status
               break;

#if defined (DUMP)
            case TXa_SLASH:                     // toggle trace value
               switch (TxTrLevel)
               {
                  case 0:  TxTrLevel = 1;   break;
                  case 1:  TxTrLevel = 100; break;
                  default: TxTrLevel = 0;   break;
               }
               break;
#endif
            default:                            // route some keys to focus-win
               #if defined (USEWINDOWING)
               if (txwa->desktop != NULL)       // if windowing is active
               {
                  switch (key)                  // do not send critical keys
                  {
                     case TXk_ENTER:
                        break;

                     default:
                        txwSendMsg((TXWHANDLE) txwa->focus, TXWM_CHAR, 0, key);
                        break;
                  }
               }
               #endif                           // USEWINDOWING
               break;
         }
      }
   }
   return( aborting);
}                                               // end 'TxAbort'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Display a message using format string and ask confirmation
/*****************************************************************************/
BOOL TxConfirm                                  // RET   Confirmed
(
   ULONG               helpid,                  // IN    helpid confirmation
   char               *fmt,                     // IN    format string (printf)
   ...                                          // IN    variable arguments
)
{
   BOOL                cf = TRUE;               // return value
   TX1K                text;
   int                 reply = 'n';
   va_list             varargs;
   char              **mText;                   // formatted text for display
   int                 ll;                      // real max line length
   int                 lines;                   // nr of lines
   SCREEN_STATE        sa = screen_act;

   if ((ll = TxScreenCols() -12) < 20)          // ScreenCols will be 0 if
   {                                            // output is redirected!
      ll = 20;
   }
   if ((txwa->desktop != NULL) && (!TxaExeSwitch('p')))
   {
      screen_act = SCREEN_OFF;                  // no regular text if windowed
   }                                            // and not 'pedantic mode'
   else
   {
      screen_act = SCREEN_ON;                   // force screen for confirm
   }
   va_start(varargs, fmt);
   vsprintf( text, fmt, varargs);               // format the message

   mText = txString2Text( text, &ll, &lines);   // convert to multi-line
   TxPrint( "\n");                              // force empty line before
   TxShowTxt(  mText);                          // display in scrollbuffer
   txFreeText( mText);                          // free the text memory

   #if defined (USEWINDOWING)
   if (txwa->desktop != NULL)                   // there is a desktop
   {
      char            *yn;

      TxStripAnsiCodes( text);                  // remove colors for popup
      if ((yn = strstr( text, "[Y/N]")) != NULL)
      {
         *yn = '\0';                            // remove [y/n] part
      }
      switch (txwMessageBox( TXHWND_DESKTOP, TXHWND_DESKTOP,
                             text, txConfirmTitle, helpid,
                             TXMB_YESNOCANCEL | TXMB_MOVEABLE |
                             TXMB_HCENTER     | TXMB_VCENTER) )
      {
         case TXMBID_OK:                        // F4=OK
         case TXMBID_YES:
            reply = 'y';                        // for logging only
            break;

         case TXMBID_CANCEL:
            TxSetPendingAbort();
            TxPrint( "Abort");
            reply = '!';
         default:
            cf = FALSE;                         // not confirmed
            break;
      }
   }
   else                                         // use non-windowed interface
   #endif                                       // USEWINDOWING
   {
      if (TxAbort())
      {
         reply = TXK_ESCAPE;
      }
      else
      {
         reply = getch();                       // wait for next keystroke
      }
      switch (reply)
      {
         case 'y': case 'Y':
         case 'j': case 'J':
            break;

         case TXK_ESCAPE:
            TxSetPendingAbort();
            TxPrint( "Abort");
            reply = '!';
         default:
            cf = FALSE;                         // not confirmed
            break;
      }
   }
   TxPrint("%c\n", reply);

   screen_act = sa;                             // restore screen state
   return (cf);
}                                               // end 'TxConfirm'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Display a message or warning using format string and wait for OK (or ESC)
/*****************************************************************************/
BOOL TxMessage                                  // RET   OK, not escaped
(
   BOOL                acknowledge,             // IN    wait for acknowledge
   ULONG               helpid,                  // IN    helpid confirmation
   char               *fmt,                     // IN    format string (printf)
   ...                                          // IN    variable arguments
)
{
   BOOL                rc = TRUE;               // return value
   TX1K                text;
   va_list             varargs;
   char              **mText;                   // formatted text for display
   int                 ll;                      // real max line length
   int                 lines;                   // nr of lines
   SCREEN_STATE        sa = screen_act;

   if ((ll = TxScreenCols() -4) < 20)           // ScreenCols will be 0 if
   {                                            // output is redirected!
      ll = 20;
   }
   if ((txwa->desktop != NULL) && (acknowledge) && (!TxaExeSwitch('p')))
   {
      screen_act = SCREEN_OFF;                  // no regular text if windowed
   }                                            // and not 'pedantic mode'
   else if (acknowledge)
   {
      screen_act = SCREEN_ON;                   // force screen on for confirm
   }
   va_start(varargs, fmt);
   vsprintf( text, fmt, varargs);               // format the message

   mText = txString2Text( text, &ll, &lines);   // convert to multi-line
   TxPrint( "\n");                              // force empty line before
   TxShowTxt(  mText);                          // display in scrollbuffer
   txFreeText( mText);                          // free the text memory
   TxPrint("\n");

   if (acknowledge)
   {
      #if defined (USEWINDOWING)
      if (txwa->desktop != NULL)                // there is a desktop
      {
         TxStripAnsiCodes( text);               // remove colors for popup
         if (txwMessageBox( TXHWND_DESKTOP, TXHWND_DESKTOP,
                            text, txMsgWarnTitle, helpid,
                            TXMB_OK      | TXMB_MOVEABLE |
                            TXMB_VCENTER | TXMB_HCENTER) != TXMBID_OK)
         {
            rc = FALSE;
         }
         TxCancelAbort();                       // avoid aborting on Escape ...
      }
      else                                      // use non-windowed interface
      #endif
      {
         TxPrint( "Press [Space-bar] to continue ...\n");
         if (getch() == TXK_ESCAPE)             // wait for next keystroke
         {
            rc = FALSE;
         }
      }
   }
   screen_act = sa;                             // restore screen state
   return (rc);
}                                               // end 'TxMessage'
/*---------------------------------------------------------------------------*/

