//
// CRC32 implementation, compatible with OS/2 LVM data-structure checksums
//
// Author: J. van Wijk
//
//

#include <txlib.h>                              // TxLib interface

#define DFS_LVM_CRC_RANGE  256
#define DFS_LVM_CRC_POLY   0xEDB88320L
#define DFS_LVM_CRC_INIT   0xFFFFFFFFL

static ULONG          *crcTable = NULL;         // for LVM type CRC

// Allocate and prepare the CRC-table needed for LVM CRC calculation
static void TxCreateLvmCrcTable
(
   void
);


/*****************************************************************************/
// Allocate and prepare the CRC-table needed for LVM CRC calculation
/*****************************************************************************/
static void TxCreateLvmCrcTable
(
   void
)
{
   crcTable = TxAlloc(DFS_LVM_CRC_RANGE, sizeof(ULONG));
   if (crcTable != NULL)
   {
      ULONG            i;
      ULONG            j;
      ULONG            value;

      for ( i = 0; i < DFS_LVM_CRC_RANGE ; i++ )
      {
        value = i;

        for ( j = 8 ; j > 0; j-- )
        {
           if (value & 1)
           {
              value = (value >> 1) ^ DFS_LVM_CRC_POLY;
           }
           else
           {
              value >>= 1;
           }
        }
        crcTable[i] = value;
      }
   }
}                                               // end 'TxCreateLvmCrcTable'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Calculate 32-bit CRC value using LVM compatible algorithm and polynom
/*****************************************************************************/
ULONG TxCalculateLvmCrc
(
   char               *area,                    // IN    data area needing CRC
   ULONG               size                     // IN    size of the data area
)
{
   ULONG               rc = DFS_LVM_CRC_INIT;   // function return, initial

   if (crcTable == NULL)
   {
      TxCreateLvmCrcTable();
   }
   if (crcTable != NULL)
   {
      char            *this = area;             // current byte handled
      ULONG            t1;
      ULONG            t2;                      // intermediate values
      ULONG            i;

      for (this = area, i = 0; i < size; i++, this++)
      {
        t1 = (rc >> 8) & 0xffffff;
        t2 = crcTable[ (rc ^ (ULONG) *this) & (ULONG) 0xff];
        rc = t1 ^ t2;
      }
   }
   return (rc);
}                                               // end 'TxCalculateLvmCrc'
/*---------------------------------------------------------------------------*/

