// TxLib filesystem functions, file-exist section
//

#include <txlib.h>                              // TxLib interface

#include <sys/stat.h>                           // for low level stuff


/*****************************************************************************/
// Create empty file with specified path/name, prompt to replace existing
/*****************************************************************************/
ULONG TxCreateEmptyFile
(
   char               *fname,                   // IN    path and filename
   BOOL                prompt                   // IN    prompt on replace
)
{
   ULONG               rc = NO_ERROR;           // function return
   FILE               *fp;

   ENTER();
   TRACES(( "Fname:'%s'  prompt:%s\n", fname, (prompt) ? "YES" : "NO"));

   if (prompt && (TxFileExists( fname)))
   {
      if (!TxConfirm( 0, "File '%s' exists, replace ? [Y/N] : ", fname))
      {
         rc = TX_ABORTED;
      }
   }
   if (rc == NO_ERROR)
   {
      if ((fp = fopen( fname, "w")) != NULL)
      {
         fclose( fp);
      }
      else
      {
         rc = TX_ACCESS_DENIED;
      }
   }
   RETURN (rc);
}                                               // end 'TxCreateEmptyFile'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Test if exact (path+) filename is accessible; supports > 2GiB files
/*****************************************************************************/
BOOL TxFileExists                               // RET   file is accessible
(
   char               *fname                    // IN    filename string
)
{
   BOOL                rc = FALSE;              // function return
   TXHFILE             fh = 0;

   #if defined (DEV32)                          // test large file support APIs
      ULONG         action;                     // action taken on open
      TXF_OS2LFAPI  api;                        // large file API

      if (TxLargeFileApiOS2( &api))
      {
         (api.DosOpenLarge)( fname, &fh, &action, 0, FILE_NORMAL, FILE_OPEN,
                             OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE, NULL);
      }
      else
      {
         DosOpen( fname, &fh, &action, 0, FILE_NORMAL, FILE_OPEN,
                  OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE, NULL);
      }
   #elif defined (WIN32)
      fh = CreateFile( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   #elif defined (LINUX)
      fh = open( fname, O_RDONLY);
   #else
      fh = fopen( fname, "r");
   #endif

   if ((fh != 0) && (fh != ((TXHFILE) -1)))
   {
      (void) TxClose( fh);
      rc = TRUE;
   }
   return( rc);
}                                               // end 'TxFileExists'
/*---------------------------------------------------------------------------*/

