// TxLib file functions, generic Seek and determine filesize
//

#include <txlib.h>                              // TxLib interface

#include <sys/stat.h>                           // for low level stuff


/*****************************************************************************/
// Test if exact (path+) filename is accessible; determine size > 2GiB files
/*****************************************************************************/
BOOL TxFileSize                                 // RET   file exists
(
   char               *fname,                   // IN    filename string
   LLONG              *size                     // OUT   filesize or NULL
)
{
   BOOL                rc = FALSE;              // function return
   TXHFILE             fh = 0;
   #if   defined (DEV32)
      ULONG            action;                  // action taken on open
      TXF_OS2LFAPI     api;                     // large file API
   #elif defined (WIN32)
      ULONG            hi = 0;                  // hi part of size
      ULONG            lo = 0;                  // lo part of size
   #elif defined (LINUX)
   #else
   #endif

   ENTER();

   #if defined (DEV32)                          // test large file support APIs
      if (TxLargeFileApiOS2( &api))
      {
         (api.DosOpenLarge)( fname, &fh, &action, 0, FILE_NORMAL, FILE_OPEN,
                             OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE, NULL);
      }
      else
      {
         DosOpen( fname, &fh, &action, 0, FILE_NORMAL, FILE_OPEN,
                  OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE, NULL);
      }
   #elif defined (WIN32)
      fh = CreateFile( fname, GENERIC_READ, FILE_SHARE_READ, NULL,
                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   #elif defined (LINUX)
      fh = open( fname, O_RDONLY | O_LARGEFILE);
   #else
      fh = fopen( fname, "r");
   #endif

   if ((fh != 0) && (fh != ((TXHFILE) -1)))
   {
      rc = TRUE;

      if (size != NULL)
      {
         *size  = 0;                            // initialize to zero size

         #if   defined (DEV32)
            if (TxLargeFileApiOS2( &api))
            {
               (api.DosSeekLarge)( fh, 0, FILE_END, size);
            }
            else
            {
               DosSetFilePtr( fh, 0, FILE_END,  (ULONG *) size);
            }
         #elif defined (WIN32)
            lo    = GetFileSize( fh, &hi);
            *size = (((LLONG) hi) << 32) + lo;

            TxPrint( "TxFileSize:  hi = %8.8lx lo = %8.8lx\n", hi, lo);
            TxPrint( "Size: %16.16llX = %lld bytes\n", *size, *size);
         #elif defined (LINUX)
             _llseek( fh, 0, 0, size, SEEK_END);
         #else
            fseek( fh, 0, SEEK_END);
            *size = (LLONG) ftell( fh);
         #endif
         TRACES(( "Size: %16.16llX = %lld bytes\n", *size, *size));
      }
      (void) TxClose( fh);
   }
   BRETURN( rc);
}                                               // end 'TxFileSize'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Seek to specified position in open file (platform specific fseek)
/*****************************************************************************/
ULONG TxFileSeek
(
   TXHFILE             fh,                      // IN    file handle
   LLONG               offset,                  // IN    seek offset
   int                 whence                   // IN    seek reference
)
{
   ULONG               rc = NO_ERROR;           // function return
   #if   defined (DEV32)
      TXF_OS2LFAPI     api;                     // large file API
      LLONG            current;
   #elif defined (WIN32)
      ULONG            hi = 0;                  // High part of seek offset
      ULONG            lo = 0;                  // Low  part of seek offset
   #elif defined (LINUX)
      LLONG            ll;
   #else
   #endif

   ENTER();

   #if   defined (DEV32)
      if (TxLargeFileApiOS2( &api))
      {
         (api.DosSeekLarge)(    fh, offset, (ULONG) whence, &current);
      }
      else
      {
         DosSetFilePtr( fh, (ULONG) offset, (ULONG) whence,  (ULONG *) &current);
      }
   #elif defined (WIN32)
      lo = offset & 0xffffffff;                 // Note: no ...PointerEx() use
      hi = offset >> 32;                        // to allow running on NT4!

      if (SetFilePointer( fh, lo, (PLONG) &hi, (DWORD) whence) == 0xffffffff)
      {
         rc = GetLastError();
      }
   #elif defined (LINUX)
      if ( _llseek( fh, (offset >> 32), (offset & 0xffffffff), &ll, whence) == -1)
      {
         rc = TxRcFromErrno( errno);
      }
   #else
      fseek( fh, (LONG) offset, whence);
   #endif
   RETURN (rc);
}                                               // end 'TxFileSeek'
/*---------------------------------------------------------------------------*/

