// TxLib filesystem functions, find file/path section
//

#include <txlib.h>                              // TxLib interface


/*****************************************************************************/
// Find file in a path specified by environment variable (like PATH)
/*****************************************************************************/
char *TxFindByPath                              // RET   ptr to filename
(
   char               *fname,                   // IN    filename string
   char               *path,                    // IN    env-var with path
   char               *found                    // OUT   found file
)
{
   char               *rc  = NULL;
   char               *p;
   char               *this;
   TX1K                env;
   TXLN                try;
   FILE               *fp;
   char               *ep;

   ENTER();
   TRACES(( "Find '%s' in paths specified by: '%s'\n", fname, path));

   if ((ep = getenv( path)) != NULL)
   {
      strcpy( env, getenv( path));
      TRACES(( "path is: '%s'\n", env));
      p = env;
      do
      {
         this = p;
         p = strchr( p, FS_ENVP_SEP);
         if (p != NULL)
         {
            *p++ = '\0';
         }
         strcpy( try, this);
         strcat( try, FS_PATH_STR);
         strcat( try, fname);
         TRACES(( "Trying fspec: '%s'\n", try));
         if ((fp = fopen( try, "r")) != NULL)
         {
            fclose( fp);
            strcpy( found, try);
            rc = found;
         }
      } while ((p != NULL) && (rc == NULL));
   }
   RETURN( rc);
}                                               // end 'TxFindByPath'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Find file in current-dir, exe-dir or PATH, and when found, open for read
/*****************************************************************************/
FILE *TxFindAndOpenFile                         // RET   opened file handle
(
   char               *fname,                   // IN    filename string
   char               *path,                    // IN    env-var with path
   char               *found                    // OUT   found file
)
{
   TXLN                try;
   FILE               *fp;
   char               *pp;

   ENTER();

   TRACES(( "Find file: '%s' using path: '%s'\n", fname, path));

   strcpy( try, fname);
   if ((fp = fopen( try, "r")) == NULL)         // try full-spec/current dir
   {
      TRACES(( "FAIL in current dir or absolute path: '%s'\n", try));
      strcpy( try, TxaExeArgv(0));
      if ((pp = strrchr( try, FS_PATH_SEP)) != NULL)
      {
         *(pp +1) = 0;                          // try executable directory
      }
      else                                      // if no executable DIR known,
      {
         strcpy( try, FS_PATH_STR);             // try root directory
      }
      strcat( try, fname);                      // append fname

      if ((fp = fopen( try, "r")) == NULL)      // not here either, try PATH
      {
         TRACES(( "FAIL in EXE-directory or root:'%s'\n", try));
         if (TxFindByPath( fname, "PATH", try) != NULL)
         {
            fp = fopen( try, "r");
         }
         else
         {
            TRACES(( "FAIL in paths for env-var: '%s'\n", "PATH"));
         }
      }
   }
   if (fp!= NULL)                               // found the file
   {
      strcpy( found, try);                      // set the found filename
      TRACES(("found file: '%s'\n", found));
   }                                            // and return the (read) fp
   RETURN (fp);
}                                               // end 'TxFindAndOpenFile'
/*---------------------------------------------------------------------------*/

