//
// Filesystem attribute handling functions
//
// Author: J. van Wijk
//
// JvW  13-06-2002 Initial version, split off from TXCON

#include <txlib.h>


// REXX value = d2x(((369*365+89)*86400)*10000000)
#define  TXNTM_LO    ((ULONG) 0xd53e8000)       // delta between 1-1-1601 base
#define  TXNTM_HI    ((ULONG) 0x019db1de)       // and ANSI 1-1-1970 (100 ns)

#define  TXNTM_HI_M  ((double) 65536 * (double) (65536))
#define  TXNTM_NS_D  ((double) 10000000)        // 100ns slots per second


/*****************************************************************************/
// Translate attributes (FAT) to 6-position string format
/*****************************************************************************/
void txFileAttr2String
(
   ULONG               data,                    // IN    data
   char               *attrib                   // OUT   attribute string[6]
)
{
   strcpy( attrib, "-----");
   if (data & FATTR_ARCHIVED)  attrib[0] = 'A';
   if (data & FATTR_DIRECTORY) attrib[1] = 'D';
   if (data & FATTR_HIDDEN)    attrib[2] = 'H';
   if (data & FATTR_READONLY)  attrib[3] = 'R';
   if (data & FATTR_SYSTEM)    attrib[4] = 'S';
}                                               // end 'txFileAttr2String'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Convert Windows-NT/W2K/XP filetime (64 bit) to compiler time_t value
/*****************************************************************************/
time_t txWinFileTime2t                          // RET   time_t value
(
   NTIME              *nt,                      // IN    ptr NT time value
   LONG                TimeZone                 // IN    TZ offset to GMT (min)
)
{
   ULONG               lo;
   ULONG               hi;
   double              tstamp;

   ENTER();

   TRACES(("NT time value 64bit: %8.8lx %8.8lx\n", nt->hi, nt->lo));
   lo = nt->lo - TXNTM_LO;
   hi = nt->hi - TXNTM_HI;                        // subtract fixed 1-1-1970
   if ( nt->lo < TXNTM_LO)
   {
      hi--;                                     // adjust for borrow-1
   }
   tstamp  = ((double) lo + ((double) hi * TXNTM_HI_M)) / TXNTM_NS_D;
   tstamp += ((double) TimeZone * 60);          // correct for timezone

   TRACES(("Seconds since 1970 : %lf\n", tstamp));
   RETURN( (time_t) tstamp);
}                                               // end 'txWinFileTime2t'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Convert OS/2 and eCS filetime (2 * USHORT) to compiler time_t value
/*****************************************************************************/
time_t txOS2FileTime2t                          // RET   time_t value
(
   USHORT             *pd,                      // IN    USHORT OS2 coded date
   USHORT             *pt                       // IN    USHORT OS2 coded time
)
{
   double              tstamp = 0;
   S_DATE             *date   = (S_DATE *) pd;
   S_TIME             *time   = (S_TIME *) pt;
   long                md = 0;                  // days difference from 1970
   long                y,m;                     // year and month numbers
   long                leaps;

   ENTER();

   TRACES(("OS2 date %4.4hx time %4.4hx\n", *pd, *pt));

   y = date->year + 1980;                       // real year
   leaps = (y - 1970) / 4 -1;                   // nr of leap-years since ...
   md = ((y - 1970) * 365) + leaps + date->day; // days since 1-1-1970
   for (m = 1; m < date->month; m++)
   {
      switch (m)
      {
         case 2:                           md += ((y%4)==0) ? 29 : 28; break;
         case 4: case 6: case 9: case 11:  md +=                   30; break;
         default:                          md +=                   31; break;
      }
   }
   tstamp = ((double) ((((md * 24) +  time->hours)  * 60) + time->minutes) *
             (double) 60) + (double) (time->twosecs * 2);

   TRACES(("Seconds since 1970 : %lf\n", tstamp));
   RETURN( (time_t) tstamp);
}                                               // end 'txOS2FileTime2t'
/*---------------------------------------------------------------------------*/

