// file-logging facilities
//
// Author: J. van Wijk
//
// JvW  26-12-96   Split-off from ansilog

#include <txlib.h>

static  FILE          *log_handle = 0;
static  TXLN           log_fname;
static  BOOL           log_7bit   = TRUE;
static  BOOL           log_reopen = FALSE;      // reopen on each line

/*****************************************************************************/
// Close existing and if specified, open new logfile
/*****************************************************************************/
BOOL TxAppendToLogFile                          // RET   logfile opened
(
   char                *fname                   // IN    name of (new) logfile
)
{
   if (log_handle != 0)                         // close old one
   {
      TxPrint("Closing logfile   : '%s'\n", log_fname);
      fclose(log_handle);
      log_handle = 0;
   }
   if (fname && strlen(fname))
   {
      if (strcmp( fname, "."))                  // not 'current one' ?
      {
         strcpy( log_fname, fname);             // use new supplied name
         TxFnameExtension( log_fname, "log");
      }
      if ((log_handle = fopen(log_fname, "a")) == 0)
      {
         TxPrint("Error opening log : '%s'\n", log_fname);
      }
      else
      {
         if (TxaOptUnSet('7'))                  // NO strip to 7-bit ASCII
         {
            log_7bit = FALSE;
         }
         if (TxaExeArgc() != 0)                 // Switches already parsed ?
         {                                      // (-q = quiet switch to come)
            TxPrint("Appending to log  : '%s' (%s-bit ASCII)\n",
                      log_fname, (log_7bit) ? "7" : "8");
         }
      }
   }
   return( log_handle != 0);
}                                               // end 'TxAppendToLogFile'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Set log close/reopen on each TxPrint on or off
/*****************************************************************************/
void TxSetLogReOpen
(
   BOOL                reopen                   // IN    log reopen on
)
{
   log_reopen = reopen;
}                                               // end 'TxSetLogReOpen'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Close existing and reopen (implementing a brute-force flush)
/*****************************************************************************/
void TxCloseReopenLogFile
(
   void
)
{
   static BOOL         warn = TRUE;

   if (log_handle != 0)
   {
      fclose(log_handle);
      if ((log_handle = fopen(log_fname, "a")) == 0)
      {
         if (warn)
         {
            TxPrint("\nError re-opening  : '%s'\n", log_fname);
            warn = FALSE;                       // I will say this only once!
         }
      }
   }
}                                               // end 'TxCloseReopenLogFile'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Signal logfile active
/*****************************************************************************/
FILE *TxQueryLogFile                            // RET   logfile active
(
   BOOL               *ascii7bit,               // OUT   Use 7-bit ASCII only
   BOOL               *reopenlog                // OUT   reopen logfile
)                                               //       NULL if not wanted
{
   if (ascii7bit != NULL)
   {
      *ascii7bit  = log_7bit;
   }
   if (reopenlog != NULL)
   {
      *reopenlog  = log_reopen;
   }
   return( log_handle);
}                                               // end 'TxQueryLogFile'
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
// Get name for current logfile, if any
/*****************************************************************************/
char *TxQueryLogName                            // RET   filename or NULL
(
   void
)                                               //       (NULL if not wanted)
{
   char               *rc = NULL;

   if (log_handle != 0)
   {
      rc = log_fname;
   }
   return (rc);
}                                               // end 'TxQueryLogName'
/*---------------------------------------------------------------------------*/

