// Generic TX string functions
//
// Author: J. van Wijk
//
// JvW  24-07-2005 Initial version, split off from TXUTIL

#include <txlib.h>

/*****************************************************************************/
// Replace specfied character by another in whole string, count replacements
/*****************************************************************************/
int TxRepl                                      // RET   nr of replacements
(
   char               *str,                     // INOUT string to convert
   char                old,                     // IN    old char, to replace
   char                new                      // IN    new char
)
{
   int                 rc = 0;

   while (*str)
   {
      if (*str == old)
      {
         *str = new;
         rc++;
      }
      str++;
   }
   return (rc);
}                                               // end 'TxRepl'
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
// Strip leading/trailing characters from a string, dest and source can be same
/*****************************************************************************/
int TxStrip                                     // RET   nr of stripped chars
(
   char               *dest,                    // OUT   destination string
   char               *source,                  // IN    source string
   char                lead,                    // IN    leading  chars to strip
   char                trail                    // IN    trailing chars to strip
)
{
   int                 rc = 0;
   char               *s = dest;

   while ((*source) && (*source == lead))
   {
      source++;
      rc++;
   }
   while (*source)
   {
      *s++ = *source++;
   }
   *s = 0;
   while ((s-- > dest) && (*s == trail))
   {
      *s = 0;
      rc++;
   }
   return (rc);
}                                               // end 'TxStrip'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Pad/Clip a string to specified length, concatenating or removing characters
/*****************************************************************************/
int TxPClip                                     // RET   +:padded -:removed
(
   char               *str,                     // INOUT destination string
   int                 size,                    // IN    requested length
   char                pad                      // IN    trailing chars to pad
)
{
   int                 pos = strlen(str);
   int                 rc  = size - pos;

   str[ size] = 0;                              // terminate at wanted length

   while (pos < size)
   {
      str[pos] = pad;
      pos++;
   }
   return (rc);
}                                               // end 'TxPClip'
/*---------------------------------------------------------------------------*/

