//
// TX string number handling functions
//
// Author: J. van Wijk
//
// JvW  24-07-2005 Initial version, split off from TXUTIL

#include <txlib.h>


/*****************************************************************************/
// Test if string represents a valid hex number, allow 0x prefix and ,unit
/*****************************************************************************/
BOOL txIsValidHex                               // RET   string is valid hex
(
   char               *num                      // IN    candidate string
)
{
   BOOL                rc = TRUE;               // function return
   char               *s  = num;

   if (strnicmp( s, "0x", 2) == 0)
   {
      s += 2;
   }
   while ((*s) && (*s != ','))                  // until end or unit separator
   {
      if (isxdigit(*s))
      {
         s++;
      }
      else
      {
         rc = FALSE;
         break;
      }
   }
   return (rc);
}                                               // end 'txIsValidHex'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Test if string represents a valid decimal nr, allow 0t/0n prefix and ,unit
/*****************************************************************************/
BOOL txIsValidDec                               // RET   string is not decimal
(
   char               *num                      // IN    candidate string
)
{
   BOOL                rc = TRUE;               // function return
   char               *s  = num;

   if ((strnicmp( s, "0t", 2) == 0) ||
       (strnicmp( s, "0n", 2) == 0)  )
   {
      s += 2;
   }
   while ((*s) && (*s != ','))                  // until end or unit separator
   {
      if (isdigit(*s))
      {
         s++;
      }
      else
      {
         rc = FALSE;
         break;
      }
   }
   return (rc);
}                                               // end 'txIsValidDec'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Test if string starts with a valid MCS prefix 0x 0n 0t 0x (case insensitive)
/*****************************************************************************/
BOOL txHasMcsPrefix                             // RET   string has MCS prefix
(
   char               *num                      // IN    candidate string
)
{
   BOOL                rc = FALSE;              // function return

   ENTER();
   TRACES(("num string: '%s'\n", num));

   if (num[0] == '0')
   {
      if (strchr("NnOoTtXx", num[1]) != NULL)
      {
         rc = TRUE;
      }
   }
   BRETURN (rc);
}                                               // end 'txHasMcsPrefix'
/*---------------------------------------------------------------------------*/

