//
// TX string to TX-text conversion and handling functions
//
// Author: J. van Wijk
//
// JvW  24-07-2005 Initial version, split off from TXUTIL

#include <txlib.h>


/*****************************************************************************/
// Convert C-string to dynamically allocated TXLIB text structure
/*****************************************************************************/
char  **txString2Text                           // RET   TXLIB text structure
(
   char               *string,                  // IN    null terminated string
   int                *maxlen,                  // INOUT line length
   int                *lines                    // OUT   nr of lines
)
{
   char              **text = NULL;             // function return
   int                 line = 0;                // nr of lines done
   int                 lpsize;                  // nr of line pointers
   int                 length;                  // length of substring
   char               *start;                   // start of line
   char               *next;                    // end of line / next

   ENTER();

   if (string != NULL)
   {
      lpsize = (strlen( string) * 3 / (*maxlen)) +9; // worst-case
      //- Note: Fails on string with many very short lines => text is clipped

      TRACES(("Strlen: %d  Maxlen: %d => Alloc %d pointers\n",
               strlen(string), *maxlen, lpsize));

      if ((text = TxAlloc( lpsize, sizeof(char *))) != NULL)
      {
         for ( start =  string, line = 0;
               start && strlen( start) && (line < lpsize -1);
               start =  next,   line++)
         {
            length = strcspn( start, "\n");
            TRACES(("found newline at pos: %d\n", length));
            if (length > *maxlen)               // no newline in sight
            {
               for ( length = *maxlen;
                    (length > 0) && (start[ length] != ' ');
                     length --)
               {
                  //- find wordwrap position (strrchr would start at end!)
               }
               TRACES(("found word-wrap space at pos: %d\n", length));
               if (length == 0)                 // one very long word, split
               {
                  length = *maxlen;
               }
            }
            TRACES(("line:%d length:%d s:'%-40.40s'\n", line, length, start));
            text[ line] = TxAlloc( 1, length +1);
            strncpy( text[ line], start, length);
            text[ line][length] = '\0';
            TRACES(("text[%d]:'%s'\n", line, text[line]));

            next = start + length;
            if (((*next == ' ') && (*(next+1) != ' ')) || (*next == '\n'))
            {
               TRACES(( "skipping one whitespace at: '%-40.40s'\n", next));
               next++;                          // skip at most 1 whitespace
            }
         }
         text[ line] = NULL;                    // explicit end-pointer

         TRACES(("line: %d\n", line));
         *lines  = line;                        // nr of lines generated
         *maxlen = 0;
         while (line--)
         {
            length = strlen( text[ line]);
            if (length > *maxlen)
            {
               *maxlen = length;                // determine maximum length
            }
         }
         TRACES(("lines: %d  with maxlen: %d\n", *lines, *maxlen));
      }
   }
   RETURN (text);
}                                               // end 'txString2Text'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Free memory allocated for dynamic TXLIB text
/*****************************************************************************/
void txFreeText
(
   char              **text                     // IN    dyn. allocated text
)
{
   char              **line;

   ENTER();

   if (text != NULL)
   {
      for (line = text; *line; line++)
      {
         TxFreeMem( *line);
      }
      TxFreeMem( text);
   }
   VRETURN ();
}                                               // end 'txFreeText'
/*---------------------------------------------------------------------------*/

