// TxLib Unicode string handling
//

#include <txlib.h>                              // TxLib interface

/*****************************************************************************/
// Append translated Unicode string-fragment to an ASCII string
/*****************************************************************************/
char *TxUnicAppend                              // RET   ptr to ASCII string
(
   USHORT             *uni,                     // IN    Unicode string part
   char               *asc,                     // INOUT ASCII string
   USHORT              len                      // IN    maximum Unicode length
)
{
   char               *pos = asc + strlen( asc); // append position
   USHORT              i;

   for (i = 0; i < len && uni[i] != 0; i++)     // until len or zero-unicode
   {
      if (uni[i] > 0xff)
      {
         if (uni[i] != 0xffff)
         {
            *pos++ = '*';                       // to be refined
         }
      }
      else
      {
         *pos++ = (char) (uni[i] & 0xff);
      }
   }
   *pos = '\0';                                 // terminate the string
   return (asc);
}                                               /* end TxUnicAppend          */
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Perform a Unicode to ASCII case-sensitive stringcompare (aka strncmp)
/*****************************************************************************/
int TxUnicStrncmp                               // RET   compare result
(
   char               *uni,                     // IN    Unicode string
   char               *asc,                     // IN    ASCII   string
   size_t              len                      // IN    max ASCII comp-length
)
{
   size_t              i;
   int                 rc = 0;

   for ( i = 0; (rc == 0) && (i < len) && *asc; i++, asc++, uni += 2)
   {
      rc = *uni - *asc;
   }
   return (rc);
}                                               /* end TxUnicStrncmp         */
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Perform a Unicode to ASCII case-insensitive stringcompare (aka strnicmp)
/*****************************************************************************/
int TxUnicStrnicmp                              // RET   compare result
(
   char               *uni,                     // IN    Unicode string
   char               *asc,                     // IN    ASCII   string
   size_t              len                      // IN    max ASCII comp-length
)
{
   size_t              i;
   int                 rc = 0;

   for ( i = 0; (rc == 0) && (i < len) && *asc; i++, asc++, uni += 2)
   {
      rc = ((*uni) & 0xdf) - ((*asc) & 0xdf);
   }
   return (rc);
}                                               /* end TxUnicStrnicmp        */
/*---------------------------------------------------------------------------*/

