//
// Generic TX utility functions
//
// Author: J. van Wijk
//
// JvW  13-06-2002 Initial version, split off from TXCON

#include <txlib.h>

static  char  txc_ascii7_t[] =                  // translate table 128..255

        "CueaaaaceeeiiiAAEaAooouuyOUcLYPfaiounNao?[]24i<>-=*|++++++|+++++"
// 128  ""

        "++++-++++++++=+++++++++++++*=||=abcpE_ut08OSmfeU=+GL[]:~o..vn2. ";
// 192  ""



/*****************************************************************************/
// Test if area consists completely of the empty byte/character ==> is empty
/*****************************************************************************/
BOOL TxAreaEmpty                                // RET   area is empty
(
   char               *area,                    // IN    area to check
   int                 size,                    // IN    size of area, bytes
   char                empty                    // IN    the empty char
)
{
   BOOL                rc = TRUE;

   while (size--)
   {
      if (*area != empty)
      {
         rc = FALSE;
         break;
      }
      area++;
   }
   return (rc);
}                                               // end 'TxAreaEmpty'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Filter 8-bit ASCII, strip to 7-bit, translate, or replace by specified char
/*****************************************************************************/
void TxAscii827                                 // translate 8-bit to 7-bit
(
   char               *str,                     // INOUT string to convert
   unsigned char       new                      // IN    new char, or 0x00
)
{
   char                ch;

   while ((ch = *str) != 0)
   {
      if ((ch & 0x80) != 0)                     // 8th bit is set
      {
         switch (new)
         {
            case TXASCII827_STRIP:
               *str = (char)(ch & 0x7f);        // simple strip off
               break;

            case TXASCII827_TRANS:
               *str = txc_ascii7_t[ch & 0x7f]; // translate using table
               break;

            default:
               *str = new;                      // replace by specified
               break;
         }
      }
      str++;
   }
}                                               // end 'TxAscii827'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Copy string to destination, replacing non printables, clip and terminate it
/*****************************************************************************/
void TxCopy
(
   char               *dest,                    // OUT   destination string
   char               *source,                  // IN    source string
   int                 len                      // IN    max length incl \0
)
{
   if (len > 0)
   {
      while ((*source) && (--len))
      {
         *dest++ = TxPrintable(*source++);
      }
      *dest = 0;
   }
}                                               // end 'TxCopy'
/*---------------------------------------------------------------------------*/

