// TX windows help manager
//
// Author: J. van Wijk
//
// JvW  12-09-2001 Initial version

#include <txlib.h>                              // public interface
#include <txwpriv.h>                            // private window interface

#define TXWH_IDCHAR    '#'                      // helpid character in text

#define TXWH_INFILE    0x00000000               // item is in a file
#define TXWH_INMEM     0x00000001               // item is in memory

#define TXWH_HELPHELP  0xfffffffe               // help-id help-on-help
#define TXWH_WID_HELP  0xff01                   // window id help screen

typedef struct txwhelp_item
{
   ULONG               helpid;                  // helpid for this item
   ULONG               linenr;                  // line-nr for the item
   ULONG               flags;                   // status flags
   union
   {
      char           **text;                    // in memory text array
      ULONG            offset;                  // offset in file for section
   };
   TXTM                title;                   // short item title
} TXWHELP_ITEM;                                 // end of struct "txwhelp_item"


static ULONG           txwh_alloc = 0;          // allocated items
static ULONG           txwh_items = 0;          // used items

static TXWHELP_ITEM   *txwh_index = NULL;       // allocated array of items

static char           *helphelp[] =
{
   "",
   " This is the help for the TxWindows userinterface and its help system",
   "",
   " TxWindows is a windowing system a bit like the operating system GUI's",
   " as MS-Windows, OS/2 or the Mac.  Unlike these GRAPHICAL interfaces,",
   " TxWindows is based on textmode displays only. This allows usage in",
   " simple systems and even from (DOS or OS/2) bootdiskettes.",
   "",
#if defined (HAVEMOUSE)
   " Allthough keyboard oriented, most windows and (menu)lists can be",
   " operated using a mouse as well. The supported mouse actions are:",
   "",
   "   Button1 click    Will move the FOCUS to the window under the mouse",
   "   (left)           pointer when possible, and if succesfull operate",
   "                    the corresponding control, like selecting from a",
   "                    (menu)list, press a button, toggle a checkbox or",
   "                    radiobutton, or any other appropriate action ...",
   "",
   "   Button1 drag     When holding the button down after clicking on a",
   "                    window title-bar, the whole window can be moved",
   "                    by dragging the mouse-cursor. It will be resized",
   "                    when the drag starts from the lower-right corner",
   "",
   "   Button2 click    Will display the HELP window related to the window",
   "   (right)          window under the mouse (context sensitive)",
   "",
   "   Button3 click    Available to application, no action by the UI itself",
   "                    Button-3 is often combined with the mouse wheel.",
   "",
   "   Key modifiers    Button actions combined with Ctrl, Alt or Shift",
   "                    keys are recognized, but currently not used by",
   "                    the UI itself. The application may use them.",
   "",
   "   Mouse chord      Mouse-chords, multiple buttons down at the same",
   "                    have no special meaning, and the behaviour when",
   "                    using them is undefined.",
   "",
   "   Wheel up/down    When supported by the mouse and mouse driver, the",
#if defined (WIN32)
   "                    wheel can be used to scroll UP/DOWN or LEFT/RIGHT",
   "                    This works by direct translation to ARROW keys and",
   "                    the PgUp/PgDn keys, so the position of the mouse",
   "                    is NOT significant. The Ctrl/Alt/Shift keys will",
   "                    modify the wheel behaviour as follows:",
   "",
   "   None             Wheel will scroll one LINE at a time (UP/DOWN key)",
   "   Shift            Wheel will scroll one PAGE at a time   (PgUp/PgDn)",
   "   Alt              Wheel will scroll one CHAR at a time  (LEFT/RIGHT)",
   "   Ctrl             Wheel will scroll the SCROLLBUFFER   (text output)",
   "                    instead of the currently active window",
   "",
   "   Combinations of the above work as expected, except for Alt+Shift",
#else
   "                    wheel can be used to scroll UP/DOWN per line or",
   "                    per page, depending on the mouse driver logic.",
#endif
   "",
   " You can activate/deactive the menu by clicking on the DESKTOP BORDER",
   " (title/footer), the top lines of the text output window, inside the",
   " command-line entryfield if the menu is up, or using the <F10> key.",
   "",
   " Many dialogs can be closed by clicking on the [X] close button in the",
   " upper right corner of the window border (same as using Escape/Cancel.",
   "",
   " In large text windows (like HELP) or large lists, scroll-indicators",
   " will appear in the right hand side of the window border, apart from",
   " using the keyboard, you can also click these to scroll up and down",
   " one line at a time, or one page at a time. The indicators used are:",
   "",
   "       Ŀ                                   Ŀ",
   "                     scroll up, 1 line    (Up)                -",
   "       text or       scroll up, 1 page    (PgUp)    text or   U",
   "       list                                         list      ",
   "       window                                       window    ",
   "       CP-437        scroll down, 1 page  (PgDn)    ASCII     D",
   "                     scroll down, 1 line  (Down)              +",
   "                                          ",
   "",
#else
   " Note: In the current implementation TxWindows can be controlled using",
   "       the keyboard only, mouse-support will be added at a later time.",
   "",
#endif
   " The screen for a TxWindows application always uses a DESKTOP window",
   " as background, other windows displayed will allways be on top of it.",
   " Most applications use a large scrolling output window (SCROLLBUFFER)",
   " that receives outputlines from the application which is scrolled up",
   " when the screen fills up. This scrollbuffer is usually very large and",
   " older output can be viewed using navigation keys like PgUp/PgDn",
   "",
   " User input is either in the form of COMMANDS that are entered in an",
   " input field usually near the bottom of the screen, or in the form of",
   " DIALOG-windows that may have fields, buttons, lists and so on ...",
   "",
   " A special form of dialog is the MENU BAR, it can be activated using",
   " the F10 function key or clicking the desktop/scrollbuffer title.",
   " The menu supports selection of actions from several pulldown menus.",
   " You can get specific help on each item in the menu by pressing the",
   " F1 functionkey while the item is selected, or by clicking it using",
   " mouse button-2 (usually the most right button).",
   "",
   "",
   " The following keys are the most important ones to control TxWindows:",
   "",
   "   Tab / Backtab Move to the next or previous selectable window, like",
   "                 between the input-field and scrollbuffer, or between",
   "                 the control-windows in an input dialog.",
   "",
   "   Esc           End or abort the current dialog, often equivalent to",
   "                 CANCELING it. Many applications also use the Esc-key",
   "                 to abort long-running commands like searching.",
   "",
   "   Enter         End current input dialog, making the entered value(s)",
   "                 available to the application for processing.",
   "",
   "   Arrow-keys    Navigate within an inputfield, scroll textlines in a",
   "                 text-window like help or move to the next or previous",
   "                 controlwindow in a dialog (buttons, fields ...)",
   "",
   "   PgUp / PgDn   Scroll textlines in a window one page at a time",
   "   Home / End    Move to begin/end of an inputfield, or output text",
   "   Ctrl-Home/End Move to first/last line of a text output window",
   "",
   "   Ctrl + Arrow  Scroll the text in the SCROLLBUFFER in the direction",
   "                 of the arrow when possible. In an inputfield the left",
   "                 and right arrow will move the cursor over one word.",
   "",
   "   Ctrl PgUp/Dn  Scoll the SCROLLBUFFER up or down, one page at a time",
   "",
   "           Note: SCROLLBUFFER movement even works from OTHER windows!",
   "",
   "",
   "   F1            Get help on the current window, inputfield, button ...",
   "                 A second F1 will give more generic help, like on the",
   "                 complete dialog, and a third will present the help on",
   "                 the TxWindows system (this text).",
   "",
   "   F3            Usually end the current dialog, canceling input, or",
   "                 end the application itself when on the commandline",
   "",
   "   F4            End the current dialog, ACCEPTING the input",
   "                 (like an [ OK ] button)",
   "",
   "   F10           Toggle the main-menu ON or OFF",
   "",
   "   F11           Present command history in a selection popup list",
   "",
   "   Alt + F2      Send COMPLETE help-file to the SCROLLBUFFER (screen)",
   "",
   "   Alt + F5      Restore resized, minimized or maximized window to its",
   "                 previous size. (initial, or just before min/maximize)",
   "",
   "   Alt + F10     Maximize the current window to the size of its parent",
   "                 client-window. (usually the SCROLLBUFFER or desktop).",
   "",
   "   Alt + F9      Minimize the current window to just its titlebar, or",
   "   F12           make it completely invisible when no titlebar present.",
   "                 This allows you to look under or behind the window.",
   "                 When the main inputfield is active, the whole DESKTOP",
   "                 will be minimized revealing the previous screen.",
   "",
   "   Alt + F12     Cycle through available SCROLLBUFFER color-schemes:",
   "                 NORMAL: White on black background, use all FG colors",
   "                 BRIGHT: White on black background, bright colors only",
   "                 INVERT: Black on white background, no bright colors",
   "                         Select the scheme with best readability",
   "",
   "   Ctrl + l      Repaint ALL windows on the desktop, can be usefull in",
   "   Ctrl + r      very rare cases to correct screen-painting problems.",
   ""
   "   Alt  + c      Change appearance of the user interface using:",
   "                   - Arrow-key UP/DOWN:    change overall color scheme",
   "                   - Arrow-key LEFT/RIGHT: change output-window colors",
   "                   - Page Up / Page Down:  change line-style scheme",
   "",
   "   Alt  + m      Move or Resize the currently displayed window:",
   "                   - Arrow-keys: Move or Resize thw window",
   "                   - Insert-key: Switch between Move and Resize",
   "",
#if defined (DUMP)
   "   Alt  + /      Cycle through available TRACE settings: OFF, TITLEBAR",
   "                 or SCREEN/LOGFILE (depending on other trace settings).",
   "",
#endif
   " Other available (function) keys might be displayed in the bottom line",
   " for a dialog window.",
   "",
   " For entryfields the following editting keys are available:",
   "",
   "    Insert              Toggle between Insert and replace mode",
   "    Delete              Delete the character at the cursor position",
   "    Backspace           Delete the character before the cursor position",
   "    Home                Move cursor to the start of the entryfield",
   "    End                 Move cursor to the end of the entryfield",
   "    Ctrl + Backspace    Clear entry-field completely, making it empty",
   "    Ctrl + E            Clear entry-field from cursor to end-of-field",
   "    Ctrl + Right Arrow  Move one word to the right in the field",
   "    Ctrl + Left  Arrow  Move one word to the left  in the field",
   "",
   " When the field supports a history buffer (like a command field would):",
   "",
   "    Ctrl + D            Delete current retrieved line from the history",
   "    Ctrl + K            Add current line to the history, no execute",
   "    Up         [prefix] Recall previous (older)  command from history",
   "    Down       [prefix] Recall next     (newer)  command from history",
   "    F11        [prefix] Show history contents in a selection popup list",
   "                        On each of these, the prefix is the part of the",
   "                        commandline BEFORE the cursor that will be used",
   "                        to FILTER the result of the requested action",
   "",
   "    Other keys are either inserted/replaced in the entryfield content,",
   "    like letters, digits and interpunction, or simply ignored",
   "",
   "",
   "",
   " The HELP system",
   "",
   " Help can be called from many places, using a help-id to specify",
   " the subject for wich help is required. This can be a specific part",
   " of the user-interface like a button or a window, or it can be a",
   " subject known to the application like a command.",
   "",
   " The help subjects can be called by pressing a [Help] button, using",
   " the <F1> function key, or any other application specified method.",
   "",
   " Once help is displayed, it can be navigated using the following keys:",
   "",
   "    PgUp / PgDn   Scroll help window up/down by one page",
   "    Left / Right  Scroll help window left/right",
   "    Up   / Down   Scroll help window up  / down",
   "    Home / End    Scroll help window to left/right margin",
   "",
   "    Esc           Quit help, return to previous active window",
   "    F1            Get more generic help, usually related to the item",
   "                  currently displayed (the 'owner' window) and when",
   "                  pressed a 3rd time, help on TxWindows (these pages)",
   "",
   "    Alt + F2      Send COMPLETE help-file to the SCROLLBUFFER (screen)",
   "                  Note that this may be thousands of lines!",
   "",
   " Since the help window is MODAL, you cannot switch to other",
   " application windows using the <Tab> key",
   "",
   "",
   NULL
};

static char           *notfound[] =
{
   "",
   " The requested help item could not be found.",
   "",
   " Please report this to your support contact for this software",
   " and include the help-id number as specified below, as well as.",
   " information on how to reproduce this possible error",
   "",
   NULL,
   NULL
};

#define TXWH_NOTFOUND_LINE 7                    // insert text at line ...

// to be refined, add sort and binary-search functions (linear search now)
// to be refined, add txwRegisterHelpFile (with multiple sections)

// Add extra space for new help items
static BOOL txwAddItemSpace
(
   ULONG               items                    // IN    nr of extra items
);

// Find a help item by its helpid (simple linear search)
static TXWHELP_ITEM *txwFindItemId              // RET   ref to item or NULL
(
   ULONG               helpid                   // IN    helpid
);


/*****************************************************************************/
// Display standard help dialog with text for specified help-id
/*****************************************************************************/
ULONG txwHelpDialog
(
   ULONG               helpid                   // IN    id for helpitem
)
{
   ULONG               rc;                      // function return
   TXWHELP_ITEM       *hi;
   char              **display = notfound;      // text to display
   ULONG               topline = 0;             // starting line number
   TXTM                title;
   TXTM                info;

   ENTER();
   TRACES(("helpid:%8.8lx = %lu\n", helpid, helpid));
   TRWINS("1");

   if (helpid != TXWH_NO_MORE_HELP)
   {
      strcpy( title, " Help item: ");           // start of help title
      if ((hi = txwFindItemId(helpid)) != NULL) // is item registred ?
      {
         display = hi->text;
         topline = hi->linenr;
         strcat( title, hi->title);
      }
      else if (helpid == TXWH_HELPHELP)         // help on help
      {
         display = helphelp;
         strcat( title, "TxWindows and the helpsystem itself");
      }
      else                                      // prepare notfound help-page
      {
         display = notfound;
         strcat( title, "not found");

         sprintf( info, " The requested help-id is: 0x%8.8lx = %lu",
                                helpid, helpid);

         notfound[ TXWH_NOTFOUND_LINE] = info;
      }
      strcat( title, " ");

      if (txwIsWindow( TXHWND_DESKTOP))         // is there a desktop ?
      {
         TXRECT           position;             // reference size/position
         TXWHANDLE        hframe;               // help frame
         TXWHANDLE        hview;                // help view text
         TXWINDOW         window;               // frame window data
         ULONG            ownerhelp = TXWH_HELPHELP;
         ULONG            focushelp = TXWH_HELPHELP;
         TXWINDOW        *win;

         txwQueryWindowRect( TXHWND_DESKTOP, FALSE, &position);
         TRECTA( "desktop size", (&position));

         if ((txwValidateHandle((TXWHANDLE) txwa->focus, &win)) != NULL)
         {
            focushelp = win->helpid;
            if ((txwValidateHandle((TXWHANDLE) txwa->focus->owner, &win)) != NULL)
            {
               ownerhelp = win->helpid;
               if ((focushelp == TXWH_HELPHELP) && (ownerhelp == TXWH_HELPHELP))
               {
                  ownerhelp = TXWH_NO_MORE_HELP; // stop recursive help on help
               }
               if (((win->client.left < 10) && (win->client.top  < 5)) ||
                   ((win->client.left + 78 < txwa->screen.right )   &&
                    (win->client.top  + 22 < txwa->screen.bottom) ) )
               {
                  position = win->client;       // owners position
               }
            }
         }
         TRECTA( "pos ", (&position));

         position.left += 2;                    // cascade the new window
         position.top  += 1;
         if (position.left + 76 < txwa->screen.right)
         {
            position.right = 76;
         }
         else
         {
            position.right = txwa->screen.right - position.left;
         }
         position.bottom = txwa->screen.bottom - position.top;
         if (position.bottom > 25)
         {
            position.bottom -= 4;               // allow some room
         }
         TRECTA( "size", (&position));

         TRACES(("focus help-id = %8.8lx = %lu\n", focushelp, focushelp));
         TRACES(("owner help-id = %8.8lx = %lu\n", ownerhelp, ownerhelp));

         txwSetupWindowData(
            position.top, position.left, position.bottom, position.right,
            TXWS_CANVAS | TXWS_CAST_SHADOW | TXWS_MOVEABLE,
            TXWH_HELPHELP,                      // help-on-help id
            ' ', ' ', TXWSCHEME_COLORS, "",  "",
            &window);
         window.dlgFocusID  = TXWH_WID_HELP;    // focus to help-text viewer
         hframe = txwCreateWindow( TXHWND_DESKTOP, TXW_FRAME, 0, 0, &window, NULL);

         txwSetupWindowData(
            0,  0, position.bottom, position.right,
            TXWS_FRAMED        |                // borders (scroll indicator)
            TXCS_CLOSE_BUTTON  |                // include a close button [X]
            TXWS_HCHILD_SIZE   |                // resize with parent
            TXWS_VCHILD_SIZE,
            ownerhelp,                          // help-on-help = owner
            ' ', ' ', TXWSCHEME_COLORS,
            title, txwstd_footer,
            &window);
         window.tv.topline = topline;
         window.tv.leftcol = 0;
         window.tv.maxtop  = TXW_INVALID;
         window.tv.maxcol  = TXW_INVALID;
         window.tv.buf     = display;
         hview = txwCreateWindow( hframe, TXW_TEXTVIEW, hframe, 0, &window,
                                  txwDefWindowProc);
         txwSetWindowUShort( hview, TXQWS_ID, TXWH_WID_HELP);

         rc = txwDlgBox( TXHWND_DESKTOP, 0, txwDefDlgProc, hframe, NULL);
      }
   }
   RETURN (rc);
}                                               // end 'txwHelpDialog'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Register a helptext of one section with multiple items
/*****************************************************************************/
ULONG txwRegisterHelpText
(
   ULONG               baseid,                  // IN    base id type/section
   char               *title,                   // IN    section title
   char               *text[]                   // IN    help text array
)
{
   ULONG               rc = NO_ERROR;           // function return
   ULONG               nr = 1;                  // nr of items / linenr
   TXWHELP_ITEM       *hi;                      // item pointer
   char              **tx;                      // text array pointer
   char               *s;

   ENTER();
   TRACES(("adding base id : %lu = 0x%8.8lx\n", baseid, baseid));

   for (tx = text; tx && *tx; tx++)
   {
      if (*tx[0] == TXWH_IDCHAR)
      {
         nr++;                                  // count items
      }
   }
   if (txwAddItemSpace( nr))
   {
      hi = &(txwh_index[ txwh_items++]);        // add default 1st item
      hi->helpid = baseid;
      hi->linenr = 0;
      hi->flags  = TXWH_INMEM;
      hi->text   = text;
      strncpy( hi->title, title, TXMAXTM);

      for ( nr = 0,  tx = text;
           (tx) && (*tx);
            nr++,    tx++)
      {
         s = *tx;                               // point to current line
         if (*s == TXWH_IDCHAR)
         {
            hi = &(txwh_index[ txwh_items++]);  // add next item from text
            hi->helpid = baseid + TxaParseNumber( s+1, TX_RADIX_STD_CLASS, NULL);
            hi->linenr = nr;
            hi->flags  = TXWH_INMEM;
            hi->text   = text;
            if ((s = strchr( *tx, ' ')) != NULL)
            {
               strncpy( hi->title, s+1, TXMAXTM);
            }
         }
      }
   }
   else
   {
      rc = TX_ALLOC_ERROR;
   }
   RETURN (rc);
}                                               // end 'txwRegisterHelpText'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Free helpmanager resources
/*****************************************************************************/
void txwTerminateHelpManager
(
   void
)
{
   ENTER();

   if (txwh_index != NULL)
   {
      free( txwh_index);
      txwh_index = NULL;
      txwh_alloc = 0;
      txwh_items = 0;
   }
   VRETURN ();
}                                              // end 'txwTerminateHelpManager'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Add extra space for new help items
/*****************************************************************************/
static BOOL txwAddItemSpace
(
   ULONG               items                    // IN    nr of extra items
)
{
   BOOL                rc = FALSE;              // function return
   char               *m = (char *) txwh_index;

   if (items > 0)
   {
      m = realloc( m, (size_t) (txwh_alloc + items) * sizeof(TXWHELP_ITEM));
      if (m != NULL)
      {
         memset( m + (         txwh_alloc * sizeof(TXWHELP_ITEM)),
                 0,  ((size_t)(items      * sizeof(TXWHELP_ITEM))) );

         txwh_index  = (TXWHELP_ITEM *) m;
         txwh_alloc += items;
         rc = TRUE;
      }
   }
   return (rc);
}                                               // end 'txwAddItemSpace'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Find a help item by its helpid (simple linear search)
/*****************************************************************************/
static TXWHELP_ITEM *txwFindItemId              // RET   ref to item or NULL
(
   ULONG               helpid                   // IN    helpid
)
{
   TXWHELP_ITEM       *rc = NULL;               // function return
   TXWHELP_ITEM       *hi;
   ULONG               i;

   ENTER();
   TRACES(("#items:% 4lu  helpid: %8.8lx = %lu\n", txwh_items, helpid, helpid));

   for (i = 0, hi = txwh_index; hi && (i < txwh_items); i++, hi++)
   {
      TRACES(("items#:% 4d  helpid: %8.8lx = %lu\n", i, hi->helpid, hi->helpid));
      if (hi->helpid == helpid)
      {
         rc = hi;                               // found the item
         break;
      }
   }
   RETURN (rc);
}                                               // end 'txwFindItemId'
/*---------------------------------------------------------------------------*/

