// TXT navigation and general function interface
//
// Author: J. van Wijk
//
// JvW  08-07-2003 Initial version, derived from DFS.H
//
#ifndef    TXT_H
   #define TXT_H

#define TXT_CMD_FAILED    ((USHORT) 901)        // Generic cmd failure code
#define TXT_ALLOC_ERROR   ((USHORT) 906)        // memory allocation error
#define TXT_PENDING       ((USHORT) 907)        // function pending
#define TXT_QUIT          ((USHORT) 909)        // quit TXT interactive


typedef struct txtinf                           // information anchor block
{
   BOOL                batch;                   // batch-mode active
   BOOL                dialogs;                 // dialogs will be used
   ULONG               verbosity;               // output verbosity
   int                 eStrategy;               // error strategy
   ULONG               retc;                    // overall return-code
   USHORT              sbWidth;                 // visible scroll-buffer width
   USHORT              sbLength;                // visible scroll-buffer length
   BOOL                autoquit;                // auto quit after fdisk/setb
   BOOL                regconfirm;              // registration conf required
   BOOL                nowindow;                // running in classic mode
   TXCELL             *sbbuf;                   // actual scroll buffer
   ULONG               sbsize;                  // actual scrollbuf size
   ULONG               sblwidth;                // actual sb linelength
   TXWHANDLE           sbwindow;                // scroll-buffer window
   TXWHANDLE           menuOwner;               // menu handling window
   BOOL                automenu;                // automatic menu activation
   BOOL                autodrop;                // automatic pulldown drop
   ULONG               menuopen;                // default drop-down menu
   ULONG               worklevel;               // when 0, activate menu
   TXSELIST           *slSchemes;               // selection list, Color schemes
   #if defined (DOS32)
      BOOL             win9x;                   // Windows-9x DosBox detected
   #endif
} TXTINF;                                       // end of struct "txtinf"

extern  TXTINF     *txta;                       // TXT anchor block

extern  char       *switchhelp[];
extern  char       *cmdhelptxt[];

#define TXT_PROFILE     "profile.txt"

// scroll to end (output), cancel-abort and execute a new command
#define txtExecEnd(c) txwSendMsg( hwnd, TXWM_CHAR, 0, TXc_END),              \
                      TxCancelAbort(),                                       \
                      txtMultiCommand((c), TRUE, TRUE, FALSE)

// cancel-abort and execute a new command
#define txtExecCmd(c) TxCancelAbort(),                                       \
                      txtMultiCommand((c), TRUE, TRUE, FALSE)


// cancel-abort and execute a new command, NO echo or prompting
#define txtExecSilent(c) TxCancelAbort(),                                    \
                         txtMultiCommand((c), FALSE, FALSE, TRUE)

//- WARNING: Never execute two txtExec... from the menu, that will
//-          cause the menu to restart after the 1st, causing hangs
//-          Use a real "multiple command" in one go instead.

#define txtBEGINWORK() {txta->worklevel++;}
#define txtENDWORK()   {                                                     \
                          if (txta->worklevel)                               \
                          {                                                  \
                             if ((--(txta->worklevel) == 0) &&               \
                                    (txta->menuOwner  != 0)  )               \
                             {                                               \
                                txwPostMsg( txta->menuOwner,                 \
                                            TXWM_COMMAND, TXTM_DEFAULT, 0);  \
                             }                                               \
                          }                                                  \
                       }


// Print TXTst logo+status, do startup checks, run startup commands + profile
ULONG txtStartupLogo                            // RET   Checks and firstcmd RC
(
   char               *firstcmd                 // IN    initial command
);


// Execute multiple txt-commands separated by # characters
ULONG txtMultiCommand
(
   char               *cmdstring,               // IN    multiple command
   BOOL                echo,                    // IN    Echo  before execute
   BOOL                prompt,                  // IN    prompt after execute
   BOOL                quiet                    // IN    screen-off during cmd
);

#endif
